from pydantic import model_validator
from typing import Union, Dict, List, Optional
from ..component import Component

class Icon(Component):
    """
    Component 类表示一个图标组件，包含图标的各种属性和方法。
    """
    component: str = "icon"  # 组件类型，默认为 "icon"
    script_url: Optional[Union[str, List[str]]] = None  # iconfont.cn 项目在线生成的 js 地址，@ant-design/icons@4.1.0 之后支持 string[] 类型
    extra_common_props: Dict[str, Union[str, int, float, bool, None]] = {}  # 给所有的 svg 图标 <Icon /> 组件设置额外的属性
    class_name: str = None  # 计算后的 svg 类名
    height: Union[int, float, str, None] = None  # svg 元素高度
    width: Union[int, float, str, None] = None  # svg 元素宽度
    type: Optional[str] = None  # 图标类型

    @model_validator(mode="after")
    def init(self):
        """
        初始化 Component 对象，并设置默认值。
        """
        self.set_key()
        return self

    def set_script_url(self, script_url: Union[str, List[str]]) -> 'Icon':
        """
        设置 iconfont.cn 项目在线生成的 js 地址。

        :param script_url: js 地址，可以是单个字符串或字符串列表
        :return: 当前 Component 对象
        """
        self.script_url = script_url
        return self

    def set_extra_common_props(self, extra_common_props: Dict[str, Union[str, int, float, bool, None]]) -> 'Icon':
        """
        设置给所有的 svg 图标 <Icon /> 组件的额外属性。

        :param extra_common_props: 额外属性字典
        :return: 当前 Component 对象
        """
        self.extra_common_props = extra_common_props
        return self

    def set_class_name(self, class_name: str) -> 'Icon':
        """
        设置计算后的 svg 类名。

        :param class_name: 类名字符串
        :return: 当前 Component 对象
        """
        self.class_name = class_name
        return self

    def set_height(self, height: Union[int, float, str, None]) -> 'Icon':
        """
        设置 svg 元素的高度。

        :param height: 高度值，可以是整数、浮点数、字符串或 None
        :return: 当前 Component 对象
        """
        self.height = height
        return self

    def set_width(self, width: Union[int, float, str, None]) -> 'Icon':
        """
        设置 svg 元素的宽度。

        :param width: 宽度值，可以是整数、浮点数、字符串或 None
        :return: 当前 Component 对象
        """
        self.width = width
        return self

    def set_type(self, icon_type: str) -> 'Icon':
        """
        设置图标类型。

        :param icon_type: 图标类型字符串
        :return: 当前 Component 对象

        "icon-database", "icon-sever", "icon-mobile", "icon-tablet", "icon-redenvelope",
        "icon-book", "icon-filedone", "icon-reconciliation", "icon-file-exception",
        "icon-filesync", "icon-filesearch", "icon-solution", "icon-fileprotect",
        "icon-file-add", "icon-file-excel", "icon-file-exclamation", "icon-file-pdf",
        "icon-file-image", "icon-file-markdown", "icon-file-unknown", "icon-file-ppt",
        "icon-file-word", "icon-file", "icon-file-zip", "icon-file-text", "icon-file-copy",
        "icon-snippets", "icon-audit", "icon-diff", "icon-Batchfolding", "icon-securityscan",
        "icon-propertysafety", "icon-insurance", "icon-alert", "icon-delete", "icon-hourglass",
        "icon-bulb", "icon-experiment", "icon-bell", "icon-trophy", "icon-rest", "icon-USB",
        "icon-skin", "icon-home", "icon-bank", "icon-filter", "icon-funnelplot", "icon-like",
        "icon-unlike", "icon-unlock", "icon-lock", "icon-customerservice", "icon-flag",
        "icon-moneycollect", "icon-medicinebox", "icon-shop", "icon-rocket", "icon-shopping",
        "icon-folder", "icon-folder-open", "icon-folder-add", "icon-deploymentunit",
        "icon-accountbook", "icon-contacts", "icon-carryout", "icon-calendar-check",
        "icon-calendar", "icon-scan", "icon-select", "icon-boxplot", "icon-build", "icon-sliders",
        "icon-laptop", "icon-barcode", "icon-camera", "icon-cluster", "icon-gateway", "icon-car",
        "icon-printer", "icon-read", "icon-cloud-server", "icon-cloud-upload", "icon-cloud",
        "icon-cloud-download", "icon-cloud-sync", "icon-video", "icon-notification", "icon-sound",
        "icon-radarchart", "icon-qrcode", "icon-fund", "icon-image", "icon-mail", "icon-table",
        "icon-idcard", "icon-creditcard", "icon-heart", "icon-block", "icon-error", "icon-star",
        "icon-gold", "icon-heatmap", "icon-wifi", "icon-attachment", "icon-edit", "icon-key",
        "icon-api", "icon-disconnect", "icon-highlight", "icon-monitor", "icon-link", "icon-man",
        "icon-percentage", "icon-pushpin", "icon-phone", "icon-shake", "icon-tag", "icon-wrench",
        "icon-tags", "icon-scissor", "icon-mr", "icon-share", "icon-branches", "icon-fork", "icon-shrink",
        "icon-arrawsalt", "icon-verticalright", "icon-verticalleft", "icon-right", "icon-left",
        "icon-up", "icon-down", "icon-fullscreen", "icon-fullscreen-exit", "icon-doubleleft",
        "icon-doubleright", "icon-arrowright", "icon-arrowup", "icon-arrowleft", "icon-arrowdown",
        "icon-upload", "icon-colum-height", "icon-vertical-align-botto", "icon-vertical-align-middl",
        "icon-totop", "icon-vertical-align-top", "icon-download", "icon-sort-descending",
        "icon-sort-ascending", "icon-fall", "icon-swap", "icon-stock", "icon-rise", "icon-indent",
        "icon-outdent", "icon-menu", "icon-unorderedlist", "icon-orderedlist", "icon-align-right",
        "icon-align-center", "icon-align-left", "icon-pic-center", "icon-pic-right", "icon-pic-left",
        "icon-bold", "icon-font-colors", "icon-exclaimination", "icon-font-size", "icon-check-circle",
        "icon-infomation", "icon-CI", "icon-line-height", "icon-Dollar", "icon-strikethrough", "icon-compass",
        "icon-underline", "icon-close-circle", "icon-number", "icon-frown", "icon-italic", "icon-info-circle",
        "icon-code", "icon-left-circle", "icon-column-width", "icon-down-circle", "icon-check", "icon-EURO",
        "icon-ellipsis", "icon-copyright", "icon-dash", "icon-minus-circle", "icon-close", "icon-meh",
        "icon-enter", "icon-plus-circle", "icon-line", "icon-play-circle", "icon-minus", "icon-question-circle",
        "icon-question", "icon-Pound", "icon-rollback", "icon-right-circle", "icon-small-dash", "icon-smile",
        "icon-pause", "icon-trademark", "icon-bg-colors", "icon-time-circle", "icon-crown", "icon-timeout",
        "icon-drag", "icon-earth", "icon-desktop", "icon-YUAN", "icon-gift", "icon-up-circle", "icon-stop",
        "icon-warning-circle", "icon-fire", "icon-sync", "icon-thunderbolt", "icon-transaction",
        "icon-alipay", "icon-undo", "icon-taobao", "icon-redo", "icon-wechat-fill", "icon-reload",
        "icon-comment", "icon-reloadtime", "icon-login", "icon-message", "icon-clear", "icon-dashboard",
        "icon-issuesclose", "icon-poweroff", "icon-logout", "icon-piechart", "icon-setting",
        "icon-eye", "icon-location", "icon-edit-square", "icon-export", "icon-save", "icon-Import",
        "icon-appstore", "icon-close-square", "icon-down-square", "icon-layout", "icon-left-square",
        "icon-play-square", "icon-control", "icon-codelibrary", "icon-detail", "icon-minus-square",
        "icon-plus-square", "icon-right-square", "icon-project", "icon-wallet", "icon-up-square",
        "icon-calculator", "icon-interation", "icon-check-square", "icon-border", "icon-border-outer",
        "icon-border-top", "icon-border-bottom", "icon-border-left", "icon-border-right", "icon-border-inner",
        "icon-border-verticle", "icon-border-horizontal", "icon-radius-bottomleft", "icon-radius-bottomright",
        "icon-radius-upleft", "icon-radius-upright", "icon-radius-setting", "icon-adduser", "icon-deleteteam",
        "icon-deleteuser", "icon-addteam", "icon-user", "icon-team", "icon-areachart", "icon-linechart",
        "icon-barchart", "icon-pointmap", "icon-container", "icon-atom", "icon-zanwutupian", "icon-safetycertificate",
        "icon-password", "icon-article", "icon-page", "icon-plugin", "icon-admin", "icon-banner"
        """
        self.type = icon_type
        return self


from typing import Dict, Any, Optional
from .base import Base


class Datetime(Base):

    component: str = "datetimeField"
    """
    组件名称
    """

    allow_clear: bool = True
    """
    是否支持清除，默认为 True
    """

    auto_focus: bool = False
    """
    自动获取焦点，默认为 False
    """

    bordered: bool = True
    """
    是否有边框，默认为 True
    """

    class_name: Optional[str] = None
    """
    自定义类名
    """

    default_value: Optional[Any] = None
    """
    默认的选中项
    """

    disabled: Optional[Any] = None
    """
    禁用状态
    """

    format: Optional[str] = "YYYY-MM-DD HH:mm:ss"
    """
    设置日期格式，为数组时支持多格式匹配，展示以第一个为准
    """

    popup_class_name: Optional[str] = None
    """
    额外的弹出日历 className
    """

    input_read_only: bool = False
    """
    设置输入框为只读，默认为 False，避免在移动设备上打开虚拟键盘
    """

    locale: Optional[Any] = None
    """
    国际化配置
    """

    mode: Optional[str] = None
    """
    日期面板的状态，取值为 'time', 'date', 'month', 'year', 'decade'
    """

    next_icon: Optional[Any] = None
    """
    自定义下一个图标
    """

    open_flag: bool = False
    """
    控制浮层显隐，默认为 False
    """

    picker: Optional[str] = None
    """
    设置选择器类型，取值为 'date', 'week', 'month', 'quarter', 'year'
    """

    placeholder: Optional[str] = "请选择"
    """
    输入框占位文本，默认为 "请选择"
    """

    placement: Optional[str] = None
    """
    浮层预设位置，取值为 'bottomLeft', 'bottomRight', 'topLeft', 'topRight'
    """

    popup_style: Optional[Any] = None
    """
    额外的弹出日历样式
    """

    prev_icon: Optional[Any] = None
    """
    自定义上一个图标
    """

    size: Optional[str] = None
    """
    输入框大小，取值为 'large', 'middle', 'small'
    """

    status: Optional[str] = None
    """
    设置校验状态，取值为 'error', 'warning'
    """

    style: Optional[Dict[str, Any]] = None
    """
    自定义样式
    """

    suffix_icon: Optional[Any] = None
    """
    自定义的选择框后缀图标
    """

    super_next_icon: Optional[Any] = None
    """
    自定义 << 切换图标
    """

    super_prev_icon: Optional[Any] = None
    """
    自定义 >> 切换图标
    """

    value: Optional[Any] = None
    """
    指定选中项，类型为 string[] | number[]
    """

    default_picker_value: Optional[str] = None
    """
    默认面板日期
    """

    show_now: bool = False
    """
    当设定了 show_time 的时候，面板是否显示“此刻”按钮，默认为 False
    """

    show_time: Optional[Any] = None
    """
    增加时间选择功能
    """

    show_today: bool = False
    """
    是否展示“今天”按钮，默认为 False
    """

    def set_allow_clear(self, allow_clear: bool):
        """
        设置是否支持清除

        Args:
            allow_clear (bool): 是否支持清除，默认 True

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.allow_clear = allow_clear
        return self

    def set_auto_focus(self, auto_focus: bool):
        """
        设置是否自动获取焦点

        Args:
            auto_focus (bool): 是否自动获取焦点，默认 False

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.auto_focus = auto_focus
        return self

    def set_bordered(self, bordered: bool):
        """
        设置是否有边框

        Args:
            bordered (bool): 是否有边框，默认 True

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.bordered = bordered
        return self

    def set_class_name(self, class_name: str):
        """
        设置自定义类名

        Args:
            class_name (str): 自定义类名

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.class_name = class_name
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def set_format(self, format: str):
        """
        设置日期格式，为数组时支持多格式匹配，展示以第一个为准

        Args:
            format (str): 日期格式字符串

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.format = format
        return self

    def set_popup_class_name(self, popup_class_name: str):
        """
        设置额外的弹出日历 className

        Args:
            popup_class_name (str): className 字符串

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.popup_class_name = popup_class_name
        return self

    def set_input_read_only(self, input_read_only: bool):
        """
        设置输入框为只读，避免在移动设备上打开虚拟键盘

        Args:
            input_read_only (bool): 是否只读

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.input_read_only = input_read_only
        return self

    def set_locale(self, locale: Any):
        """
        设置国际化配置

        Args:
            locale (Any): 国际化配置信息

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.locale = locale
        return self

    def set_mode(self, mode: str):
        """
        设置日期面板的状态

        Args:
            mode (str): 状态字符串，取值为 'time', 'date', 'month', 'year', 'decade'

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.mode = mode
        return self

    def set_next_icon(self, next_icon: Any):
        """
        设置自定义下一个图标

        Args:
            next_icon (Any): 图标信息

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.next_icon = next_icon
        return self

    def set_open_flag(self, open_flag: bool):
        """
        控制浮层显隐

        Args:
            open_flag (bool): 是否显示浮层

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.open_flag = open_flag
        return self

    def set_picker(self, picker: str):
        """
        设置选择器类型

        Args:
            picker (str): 选择器类型字符串，取值为 'date', 'week', 'month', 'quarter', 'year'

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.picker = picker
        return self

    def set_placeholder(self, placeholder: str):
        """
        设置输入框占位文本

        Args:
            placeholder (str): 占位文本

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.placeholder = placeholder
        return self

    def set_placement(self, placement: str):
        """
        设置浮层预设位置

        Args:
            placement (str): 预设位置字符串，取值为 'bottomLeft', 'bottomRight', 'topLeft', 'topRight'

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.placement = placement
        return self

    def set_popup_style(self, popup_style: Any):
        """
        设置额外的弹出日历样式

        Args:
            popup_style (Any): 样式信息

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.popup_style = popup_style
        return self

    def set_prev_icon(self, prev_icon: Any):
        """
        设置自定义上一个图标

        Args:
            prev_icon (Any): 图标信息

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.prev_icon = prev_icon
        return self

    def set_size(self, size: str):
        """
        设置输入框大小

        Args:
            size (str): 大小字符串，取值为 'large', 'middle', 'small'

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.size = size
        return self

    def set_status(self, status: str):
        """
        设置校验状态

        Args:
            status (str): 校验状态字符串，取值为 'error', 'warning'

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.status = status
        return self

    def set_suffix_icon(self, suffix_icon: Any):
        """
        设置自定义的选择框后缀图标

        Args:
            suffix_icon (Any): 图标信息

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.suffix_icon = suffix_icon
        return self

    def set_super_next_icon(self, super_next_icon: Any):
        """
        设置自定义 << 切换图标

        Args:
            super_next_icon (Any): 图标信息

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.super_next_icon = super_next_icon
        return self

    def set_super_prev_icon(self, super_prev_icon: Any):
        """
        设置自定义 >> 切换图标

        Args:
            super_prev_icon (Any): 图标信息

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.super_prev_icon = super_prev_icon
        return self

    def set_default_picker_value(self, default_picker_value: str):
        """
        设置默认面板日期

        Args:
            default_picker_value (str): 默认面板日期字符串

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.default_picker_value = default_picker_value
        return self

    def set_show_now(self, show_now: bool):
        """
        设置当设定了 show_time 的时候，面板是否显示“此刻”按钮

        Args:
            show_now (bool): 是否显示“此刻”按钮

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.show_now = show_now
        return self

    def set_show_time(self, show_time: Any):
        """
        设置增加时间选择功能

        Args:
            show_time (Any): 时间选择功能信息

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.show_time = show_time
        return self

    def set_show_today(self, show_today: bool):
        """
        设置是否展示“今天”按钮

        Args:
            show_today (bool): 是否展示“今天”按钮

        Returns:
            Component: 返回当前 Component 实例，支持链式调用
        """
        self.show_today = show_today
        return self



import uuid
from typing import List

from ..models.menu import Menu


class MenuService:
    def __init__(self, guard_name: str = "admin"):
        self.guard_name = guard_name

    def set_guard_name(self, guard_name: str):
        self.guard_name = guard_name
        return self

    async def get_list(self) -> List[Menu]:
        menus = (
            await Menu.filter(guard_name=self.guard_name, status=1)
            .order_by("sort", "id")
            .all()
        )
        return menus

    async def get_list_with_root(self):
        menus = await self.get_list()
        menus.append(Menu(id=0, pid=-1, name="根节点"))
        return menus

    async def find_parent_tree_node(self, child_pid: int):
        menus = await Menu.filter(
            guard_name=self.guard_name, id=child_pid, status=1, type__in=[1, 2, 3]
        ).all()
        if not menus:
            return menus

        result = []
        for menu in menus:
            if menu.pid != 0:
                children = await self.find_parent_tree_node(menu.pid)
                result.extend(children)
            result.append(menu)

        return result

    async def get_list_by_user_id(self, user_id: int):
        if user_id == 1:
            menus = (
                await Menu.filter(
                    guard_name=self.guard_name, status=1, type__in=[1, 2, 3]
                )
                .order_by("sort")
                .all()
            )
            return await self.menu_parser(menus)

        # role_has_menus = await CasbinService().get_user_menus(user_id)
        # TODO: 这里需要替换为实际的获取用户菜单的方法
        role_has_menus = []
        if not role_has_menus:
            return []

        menu_ids = [menu.id for menu in role_has_menus]

        menus = (
            await Menu.filter(
                guard_name=self.guard_name,
                status=1,
                id__in=menu_ids,
                type__in=[1, 2, 3],
            )
            .exclude(pid=0)
            .all()
        )

        for menu in menus:
            parent_menus = await self.find_parent_tree_node(menu.pid)
            for parent in parent_menus:
                if parent.id not in menu_ids:
                    menu_ids.append(parent.id)

        all_menus = (
            await Menu.filter(guard_name=self.guard_name, status=1, id__in=menu_ids)
            .order_by("sort")
            .all()
        )

        return await self.menu_parser(all_menus)

    async def menu_parser(self, menus):
        new_menus = []
        for menu in menus:
            menu.meta = {
                "icon": menu.icon,
                "title": menu.name,
                "order": menu.sort,
                "hide_in_menu": False if menu.visible == 1 else True,
            }
            menu.name = menu.path
            if not self.has_menu(new_menus, menu.id):
                new_menus.append(menu)

        return self.list_to_tree(new_menus, "id", "pid", "children", 0)

    def has_menu(self, menus, menu_id):
        return any(menu.id == menu_id for menu in menus)

    async def get_info_by_id(self, menu_id: int):
        menu = await Menu.filter(status=1, id=menu_id).first()
        return menu

    async def get_info_by_name(self, name: str):
        menu = await Menu.filter(status=1, name=name).first()
        return menu

    async def is_exist(self, menu_id: int):
        menu = await Menu.filter(id=menu_id).first()
        return bool(menu)

    async def get_list_by_ids(self, menu_ids):
        menus = await Menu.filter(id__in=menu_ids).all()
        return menus

    def list_to_tree(self, items, id_key, parent_key, children_key, root_id):
        tree = []
        lookup = {}

        for item in items:
            item_dict = item.__dict__.copy()
            item_dict.pop("_state", None)
            lookup[item_dict[id_key]] = item_dict
            item_dict[children_key] = []

        for item in items:
            item_dict = lookup[item.id]
            parent_id = item_dict[parent_key]
            if parent_id == root_id:
                tree.append(item_dict)
            else:
                if parent_id in lookup:
                    lookup[parent_id][children_key].append(item_dict)

        return tree



from pydantic import model_validator
from ..component import Component
from typing import Any


class ItemGroup(Component):
    component: str = "menuItemGroup"
    title: str
    items: Any

    @model_validator(mode="after")
    def init(self):
        self.set_key("DEFAULT_KEY", False)
        return self

    # 设置收缩时展示的悬浮标题
    def set_title(self, title: str) -> "ItemGroup":
        self.title = title
        return self

    # 设置按钮文字
    def set_items(self, items: Any) -> "ItemGroup":
        self.items = items
        return self



import json
from typing import Dict, List, Optional, Any
from .base import Base


class Option(Base):
    """
    表示单选按钮选项的类。

    Attributes:
        label (str): 选项显示的标签。
        value (Any): 选项对应的值。
        disabled (bool): 选项是否禁用，默认为 False。
    """

    label: str
    value: Any
    disabled: bool = False


class Radio(Base):

    component: str = "radioField"
    """
    组件名称
    """

    button_style: Optional[Any] = None
    """
    RadioButton 的风格样式，目前有描边和填色两种风格 outline | solid
    """

    default_value: Optional[Any] = None
    """
    默认选中的选项
    """

    disabled: bool = False
    """
    整组失效，默认值为 False
    """

    options: List[Option] = []
    """
    可选项数据源
    """

    option_type: Optional[str] = None
    """
    用于设置 Radio options 类型 default | button
    """

    size: Optional[str] = None
    """
    大小，只对按钮样式生效, large | middle | small
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    def get_options(self) -> List[Option]:
        """
        获取当前可选项。

        Returns:
            List[Option]: 可选项列表。
        """
        return self.options

    def build_options(
        self, items: Any, label_name: str, value_name: str
    ) -> List[Option]:
        """
        使用反射构建树结构。

        Args:
            items (Any): 包含选项数据的对象。
            label_name (str): 标签字段名。
            value_name (str): 值字段名。

        Returns:
            List[Option]: 构建好的选项列表。
        """
        options: List[Option] = []

        if not isinstance(items, list):
            return options

        for item in items:
            if hasattr(item, label_name) and hasattr(item, value_name):
                label = getattr(item, label_name)
                value = getattr(item, value_name)
                option = Option(label=label, value=value)
                options.append(option)

        return options

    def list_to_options(
        self, list_data: Any, label_name: str, value_name: str
    ) -> List[Option]:
        """
        将列表数据转换为选项列表。

        Args:
            list_data (Any): 列表数据。
            label_name (str): 标签字段名。
            value_name (str): 值字段名。

        Returns:
            List[Option]: 转换后的选项列表。
        """
        return self.build_options(list_data, label_name, value_name)

    def set_options(self, *options: Any):
        """
        设置属性，示例：[Option(value=1, label="男"), Option(value=2, label="女")]
        或者 set_options(options, "label_name", "value_name")

        Args:
            *options: 可变参数，不同数量有不同含义。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        if len(options) == 1:
            if isinstance(options[0], list) and all(
                isinstance(option, Option) for option in options[0]
            ):
                self.options = options[0]
                return self
        if len(options) == 3:
            self.options = self.list_to_options(options[0], options[1], options[2])
        return self

    def set_api(self, api: str):
        """
        设置获取数据接口。

        Args:
            api (str): 接口地址。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.api = api
        return self

    def set_button_style(self, button_style: Any):
        """
        设置 RadioButton 的风格样式。

        Args:
            button_style (Any): 风格样式信息。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.button_style = button_style
        return self

    def set_option_type(self, option_type: str):
        """
        设置 Radio options 类型。

        Args:
            option_type (str): 类型，如 'default', 'button'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.option_type = option_type
        return self

    def set_size(self, size: str):
        """
        设置控件大小。

        Args:
            size (str): 大小，如 'large', 'middle', 'small'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def get_value_enum(self) -> Dict[Any, Any]:
        """
        获取当前列值的枚举 valueEnum。

        Returns:
            Dict[Any, Any]: 列值枚举字典。
        """
        data: Dict[Any, Any] = {}
        for v in self.options:
            data[v.value] = v.label
        return data

    def get_option_label(self, value: Any) -> str:
        """
        根据 value 值获取 Option 的 Label。

        Args:
            value (Any): 要查找的 value 值。

        Returns:
            str: 对应的 Label 字符串。
        """
        labels: List[str] = []
        values: List[Any] = []

        if isinstance(value, str):
            if "[" in value or "{" in value:
                try:
                    values = json.loads(value)
                except json.JSONDecodeError:
                    pass

        if values:
            for option in self.options:
                for v in values:
                    if v == option.value:
                        labels.append(option.label)
        else:
            for option in self.options:
                if value == option.value:
                    labels.append(option.label)

        return ",".join(labels)

    def get_option_value(self, label: str) -> Any:
        """
        根据 label 值获取 Option 的 Value。

        Args:
            label (str): 要查找的 label 值。

        Returns:
            Any: 对应的 Value。
        """
        values: List[Any] = []
        get_labels = label.split(",")
        if len(get_labels) == 1:
            get_labels = label.split("，")

        if len(get_labels) > 1:
            for v in self.options:
                for get_label in get_labels:
                    if v.label == get_label:
                        values.append(v.value)
        else:
            for v in self.options:
                if v.label == label:
                    return v.value

        return values

    def get_option_labels(self) -> str:
        """
        获取 Option 的 Labels。

        Returns:
            str: 所有 Option 的 Label 组成的字符串。
        """
        labels = [option.label for option in self.options]
        return ",".join(labels)



from typing import Any, Dict, List, Optional

from pydantic import BaseModel

from .base import Base


class FieldNames(BaseModel):
    label: str
    value: str
    children: str


class TreeData(BaseModel):
    title: str
    value: Any
    children: List["TreeData"] = []
    disabled: Optional[bool] = None
    disable_checkbox: Optional[bool] = None
    selectable: Optional[bool] = None
    checkable: Optional[bool] = None


class TreeSelect(Base):
    component: Optional[str] = "treeSelectField"
    """
    组件名称
    """

    allow_clear: bool = True
    """
    可以点击清除图标删除内容
    """

    auto_clear_search_value: Optional[bool] = None
    """
    是否在选中项后清空搜索框，只在 mode 为 multiple 或 tags 时有效
    """

    bordered: Optional[bool] = None
    """
    是否有边框
    """

    default_value: Optional[Any] = None
    """
    默认选中的选项
    """

    disabled: Optional[bool] = None
    """
    整组失效
    """

    popup_class_name: Optional[str] = None
    """
    下拉菜单的 className 属性
    """

    dropdown_match_select_width: Optional[Any] = None
    """
    下拉菜单和选择器同宽。默认将设置 min-width，当值小于选择框宽度时会被忽略。false 时会关闭虚拟滚动
    """

    dropdown_style: Optional[Any] = None
    """
    下拉菜单的 style 属性
    """

    field_names: Optional[FieldNames] = None
    """
    自定义 options 中 label value children 的字段
    """

    label_in_value: Optional[bool] = None
    """
    是否把每个选项的 label 包装到 value 中，会把 Select 的 value 类型从 string 变为 { value: string, label: ReactNode } 的格式
    """

    list_height: int = 256
    """
    设置弹窗滚动高度 256
    """

    max_tag_count: int = 0
    """
    最多显示多少个 tag，响应式模式会对性能产生损耗
    """

    max_tag_placeholder: Optional[str] = None
    """
    隐藏 tag 时显示的内容
    """

    max_tag_text_length: int = 0
    """
    最大显示的 tag 文本长度
    """

    multiple: Optional[bool] = None
    """
    支持多选（当设置 treeCheckable 时自动变为 true）
    """

    not_found_content: Optional[str] = None
    """
    当下拉列表为空时显示的内容
    """

    placeholder: Optional[str] = None
    """
    选择框默认文本
    """

    placement: Optional[str] = None
    """
    选择框弹出的位置 bottomLeft bottomRight topLeft topRight
    """

    search_value: Optional[str] = None
    """
    控制搜索文本
    """

    show_arrow: Optional[bool] = None
    """
    是否显示下拉小箭头
    """

    show_search: Optional[bool] = None
    """
    配置是否可搜索
    """

    size: Optional[str] = None
    """
    选择框大小
    """

    status: Optional[str] = None
    """
    设置校验状态 'error' | 'warning'
    """

    suffix_icon: Optional[Any] = None
    """
    自定义的选择框后缀图标
    """

    switcher_icon: Optional[Any] = None
    """
    自定义树节点的展开/折叠图标
    """

    tree_checkable: Optional[bool] = None
    """
    显示 Checkbox
    """

    tree_check_strictly: Optional[bool] = None
    """
    checkable 状态下节点选择完全受控（父子节点选中状态不再关联），会使得 labelInValue 强制为 true
    """

    tree_data: List[TreeData] = []
    """
    treeNodes 数据，如果设置则不需要手动构造 TreeNode 节点（value 在整个树范围内唯一）
    """

    tree_data_simple_mode: Optional[Any] = None
    """
    使用简单格式的 treeData，具体设置参考可设置的类型 (此时 treeData 应变为这样的数据结构: [{id:1, pId:0, value:'1', title:"test1",...},...]， pId 是父节点的 id)
    """

    tree_default_expand_all: bool = True
    """
    默认展开所有树节点
    """

    tree_default_expanded_keys: Optional[List[Any]] = None
    """
    默认展开的树节点
    """

    tree_expand_action: Optional[Any] = None
    """
    点击节点 title 时的展开逻辑，可选：false | click | doubleClick
    """

    tree_expanded_keys: Optional[List[Any]] = None
    """
    设置展开的树节点
    """

    tree_icon: Optional[bool] = None
    """
    是否展示 TreeNode title 前的图标，没有默认样式，如设置为 true，需要自行定义图标相关样式
    """

    tree_line: bool = True
    """
    是否展示线条样式
    """

    tree_node_filter_prop: Optional[str] = None
    """
    输入项过滤对应的 treeNode 属性
    """

    tree_node_label_prop: Optional[str] = None
    """
    作为显示的 prop 设置
    """

    value: Optional[Any] = None
    """
    指定当前选中的条目，多选时为一个数组。（value 数组引用未变化时，Select 不会更新）
    """

    virtual: Optional[bool] = None
    """
    设置 false 时关闭虚拟滚动
    """

    style: Optional[Dict[str, Any]] = None
    """
    自定义样式
    """

    def __init__(self, **data):
        super().__init__(**data)
        self.component = "treeSelectField"
        self.colon = True
        self.label_align = "right"
        self.show_on_index = True
        self.show_on_detail = True
        self.show_on_creation = True
        self.show_on_update = True
        self.show_on_export = True
        self.show_on_import = True
        self.allow_clear = True
        self.tree_default_expand_all = True
        self.tree_line = True
        self.set_width("200px")
        self.set_key("DEFAULT_KEY", False)

    def get_options(self) -> List[TreeData]:
        """
        获取当前可选项。

        Returns:
            List[Option]: 可选项列表。
        """
        return self.tree_data

    def set_allow_clear(self, allow_clear: bool):
        self.allow_clear = allow_clear
        return self

    def set_auto_clear_search_value(self, auto_clear_search_value: bool):
        self.auto_clear_search_value = auto_clear_search_value
        return self

    def set_bordered(self, bordered: bool):
        self.bordered = bordered
        return self

    def set_popup_class_name(self, popup_class_name: str):
        self.popup_class_name = popup_class_name
        return self

    def set_dropdown_match_select_width(self, dropdown_match_select_width: Any):
        self.dropdown_match_select_width = dropdown_match_select_width
        return self

    def set_dropdown_style(self, dropdown_style: Any):
        self.dropdown_style = dropdown_style
        return self

    def set_field_names(self, field_names: FieldNames):
        self.field_names = field_names
        return self

    def set_label_in_value(self, label_in_value: bool):
        self.label_in_value = label_in_value
        return self

    def set_list_height(self, list_height: int):
        self.list_height = list_height
        return self

    def set_max_tag_count(self, max_tag_count: int):
        self.max_tag_count = max_tag_count
        return self

    def set_max_tag_placeholder(self, max_tag_placeholder: str):
        self.max_tag_placeholder = max_tag_placeholder
        return self

    def set_max_tag_text_length(self, max_tag_text_length: int):
        self.max_tag_text_length = max_tag_text_length
        return self

    def set_multiple(self, multiple: bool):
        self.multiple = multiple
        return self

    def set_not_found_content(self, not_found_content: str):
        self.not_found_content = not_found_content
        return self

    def set_placeholder(self, placeholder: str):
        self.placeholder = placeholder
        return self

    def set_placement(self, placement: str):
        self.placement = placement
        return self

    def set_search_value(self, search_value: str):
        self.search_value = search_value
        return self

    def set_show_arrow(self, show_arrow: bool):
        self.show_arrow = show_arrow
        return self

    def set_show_search(self, show_search: bool):
        self.show_search = show_search
        return self

    def set_size(self, size: str):
        self.size = size
        return self

    def set_status(self, status: str):
        self.status = status
        return self

    def set_suffix_icon(self, suffix_icon: Any):
        self.suffix_icon = suffix_icon
        return self

    def set_switcher_icon(self, switcher_icon: Any):
        self.switcher_icon = switcher_icon
        return self

    def set_tree_checkable(self, tree_checkable: bool):
        self.tree_checkable = tree_checkable
        return self

    def set_tree_check_strictly(self, tree_check_strictly: bool):
        self.tree_check_strictly = tree_check_strictly
        return self

    def build_tree(
        self,
        items: Any,
        pid: int,
        parent_key_name: str,
        title_name: str,
        value_name: str,
    ) -> List[TreeData]:
        tree = []

        for item in items:
            # 支持对象和字典两种格式
            if hasattr(item, value_name):
                value = getattr(item, value_name)
                parent_key = getattr(item, parent_key_name)
                title = getattr(item, title_name)
            elif isinstance(item, dict):
                value = item.get(value_name)
                parent_key = item.get(parent_key_name)
                title = item.get(title_name)
            else:
                # 如果既不是对象也不是字典，跳过该项
                continue

            if parent_key == pid:
                children = self.build_tree(
                    items, value, parent_key_name, title_name, value_name
                )
                option = TreeData(title=title, value=value, children=children)
                tree.append(option)

        return tree

    def list_to_tree_data(
        self,
        list_data: Any,
        root_id: int,
        parent_key_name: str,
        title_name: str,
        value_name: str,
    ) -> List[TreeData]:
        return self.build_tree(
            list_data, root_id, parent_key_name, title_name, value_name
        )

    def set_tree_data(self, *tree_data: Any):
        if len(tree_data) == 1:
            if isinstance(tree_data[0], list) and all(
                isinstance(item, TreeData) for item in tree_data[0]
            ):
                self.tree_data = tree_data[0]
                return self
        if len(tree_data) == 4:
            self.tree_data = self.list_to_tree_data(
                tree_data[0], 0, tree_data[1], tree_data[2], tree_data[3]
            )
        if len(tree_data) == 5:
            self.tree_data = self.list_to_tree_data(
                tree_data[0], tree_data[1], tree_data[2], tree_data[3], tree_data[4]
            )
        return self

    def get_tree_data(self) -> List[TreeData]:
        return self.tree_data

    def get_data(self) -> List[TreeData]:
        return self.tree_data

    def set_tree_data_simple_mode(self, tree_data_simple_mode: Any):
        self.tree_data_simple_mode = tree_data_simple_mode
        return self

    def set_tree_default_expand_all(self, tree_default_expand_all: bool):
        self.tree_default_expand_all = tree_default_expand_all
        return self

    def set_tree_default_expanded_keys(self, tree_default_expanded_keys: List[Any]):
        self.tree_default_expanded_keys = tree_default_expanded_keys
        return self

    def set_tree_expand_action(self, tree_expand_action: List[Any]):
        self.tree_expand_action = tree_expand_action
        return self

    def set_tree_expanded_keys(self, tree_expanded_keys: List[Any]):
        self.tree_expanded_keys = tree_expanded_keys
        return self

    def set_tree_icon(self, tree_icon: bool):
        self.tree_icon = tree_icon
        return self

    def set_tree_line(self, tree_line: bool):
        self.tree_line = tree_line
        return self

    def set_tree_node_filter_prop(self, tree_node_filter_prop: str):
        self.tree_node_filter_prop = tree_node_filter_prop
        return self

    def set_tree_node_label_prop(self, tree_node_label_prop: str):
        self.tree_node_label_prop = tree_node_label_prop
        return self

    def set_virtual(self, virtual: bool):
        self.virtual = virtual
        return self

    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from tortoise import fields
from tortoise.models import Model
from tortoise.transactions import in_transaction


class Department(Model):
    id = fields.IntField(pk=True)
    pid = fields.IntField(default=0)
    name = fields.CharField(max_length=500)
    sort = fields.IntField(default=0)
    status = fields.IntField(default=1)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True, null=True)

    class Meta(Model.Meta):
        table = "departments"

    @staticmethod
    async def seeder():
        async with in_transaction() as connection:
            root = (
                await Department.filter(name="夸克云科技", pid=0)
                .using_db(connection)
                .first()
            )
            if not root:
                root = await Department.create(
                    name="夸克云科技", pid=0, sort=0, status=1
                )

            # 检查子部门是否存在，不存在则创建
            centers = ["研发中心", "营销中心"]
            for name in centers:
                exists = (
                    await Department.filter(name=name, pid=root.id)
                    .using_db(connection)
                    .exists()
                )
                if not exists:
                    await Department.create(name=name, pid=root.id, sort=0, status=1)



from typing import Any
from fastapi_cache import FastAPICache
from fastapi_cache.backends.inmemory import InMemoryBackend


def init(prefix: str) -> None:
    """初始化缓存"""
    FastAPICache.init(InMemoryBackend(), prefix=prefix)


async def set(key: str, value: Any, expire: int | None = None) -> None:
    await FastAPICache.get_backend().set(key, value, expire)


async def get(key: str) -> Any:
    return await FastAPICache.get_backend().get(key)



from pydantic import Field, model_validator
from typing import Any, Dict, Optional
from ..component import Component

class Page(Component):
    component: str = Field(default="page")
    title: str = Field(..., description="标题")
    body: Any = Field(..., description="内容")
    style: Optional[Dict[str, Any]] = Field(None, description="样式")

    @model_validator(mode="after")
    def init(self):
        self.set_key("page")
        return self

    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_title(self, title: str):
        # 设置标题
        self.title = title
        return self

    def set_body(self, body: Any):
        # 设置内容
        self.body = body
        return self


from typing import Any, Dict, List

from quark import Request
from quark.component.action.action import Action
from quark.component.form.form import Form
from quark.template.action import Drawer
from quark.template.resolves_fields import ResolvesFields


class MenuCreateDrawer(Drawer):

    def __init__(self, resource: Any):

        # 资源
        self.resource = resource

        # 名称
        self.name = "创建"

        # 类型
        self.type = "primary"

        # 幽灵按钮
        self.ghost = True

        # 图标
        self.icon = "ant-design:plus-outlined"

        # 执行成功后刷新的组件
        self.reload = "table"

        # 关闭时销毁 Drawer 里的子元素
        self.destroy_on_close = True

        # 抽屉弹出层宽度
        self.width = 750

        # 设置展示位置（只在列表页显示）
        self.set_only_on_index(True)

    async def get_body(self, request: Request):
        api = await self.resource.creation_api(request)
        fields = ResolvesFields(
            request=request,
            fields=await self.resource.fields(request),
        ).creation_fields_within_components()
        data = await self.resource.before_creating(request)
        return (
            Form()
            .set_api(api)
            .set_body(fields)
            .set_layout("vertical")
            .set_initial_values(data)
            .set_key("createDrawerForm", False)
        )

    async def get_actions(self, request: Request):
        return [
            Action().set_label("取消").set_action_type("cancel"),
            Action()
            .set_submit_form("createDrawerForm")
            .set_label("提交")
            .set_with_loading(True)
            .set_reload("table")
            .set_action_type("submit")
            .set_type("primary", False),
        ]



from typing import Dict, Any, Optional, List
from .base import Base


class File(Base):

    component: str = "fileField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认选中的选项
    """

    disabled: bool = False
    """
    整组是否失效，默认为 False
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    button: str = "上传文件"
    """
    上传按钮标识，默认为 "上传文件"
    """

    limit_size: int = 2
    """
    上传文件大小限制，默认为 2
    """

    limit_type: List[str] = ["jpeg", "png", "doc", "docx"]
    """
    上传文件类型限制，默认为 ["jpeg", "png", "doc", "docx"]
    """

    limit_num: int = 3
    """
    上传文件数量限制，默认为 3
    """

    limit_wh: Dict[str, int] = {}
    """
    上传图片宽高限制
    """

    def set_limit_size(self, limit_size: int):
        """
        设置上传文件大小限制。

        Args:
            limit_size (int): 大小限制值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.limit_size = limit_size
        return self

    def set_limit_type(self, limit_type: List[str]):
        """
        设置上传文件类型限制。

        Args:
            limit_type (List[str]): 类型限制列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.limit_type = limit_type
        return self

    def set_limit_num(self, limit_num: int):
        """
        设置上传文件数量限制。

        Args:
            limit_num (int): 数量限制值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.limit_num = limit_num
        return self

    def set_button(self, text: str):
        """
        设置上传按钮的标题。

        Args:
            text (str): 按钮标题。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.button = text
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from typing import Any, Optional

from quark import Request

from ..component.form import field
from ..component.table.search import Search as TableSearch


class ResolvesSearches:

    # 请求
    request: Optional[Request] = None

    # 搜索组件
    search: Optional[TableSearch] = None

    # 搜索字段
    searches: Optional[Any] = None

    # 导出功能
    export = False

    # 导出文字
    export_text = None

    # 导出路径
    export_path = None

    def __init__(
        self,
        request: Request,
        search: Optional[Any] = None,
        searches: Optional[Any] = None,
        export: bool = False,
        export_text: str = "",
        export_path: str = "",
    ):
        self.request = request
        self.search = search
        self.searches = searches
        self.export = export
        self.export_text = export_text
        self.export_path = export_path

    def set_request(self, request) -> "ResolvesSearches":
        """设置请求"""
        self.request = request
        return self

    def set_search(self, search) -> "ResolvesSearches":
        """设置搜索组件"""
        self.search = search
        return self

    def set_searches(self, searches) -> "ResolvesSearches":
        """设置字段"""
        self.searches = searches
        return self

    def set_export(
        self, export: bool, export_text: str, export_path: str
    ) -> "ResolvesSearches":
        """设置导出功能"""
        self.export = export
        self.export_text = export_text
        self.export_path = export_path
        return self

    def index_searches(self):

        # 搜索组件
        search = self.search

        # 搜索字段
        searches = self.searches

        # 是否携带导出功能
        export = self.export
        if export:
            export_text = self.export_text  # 导出按钮文字内容
            export_path = self.export_path
            search.set_export_text(export_text).set_export_api(
                export_path.replace(
                    "{resource}", self.request.path_params.get("resource")
                )
            )

        # 解析搜索项
        for v in searches:
            # 搜索栏表单项
            item = None

            # 假设每个 search 是一个 Searcher 实例
            search_instance = v

            # 获取组件名称
            component = search_instance.get_component()

            # 获取组件的标签
            label = search_instance.get_name()

            # 获取字段名，支持数组
            name = search_instance.get_column(v)

            # 获取接口
            api = search_instance.get_api()

            # 获取属性
            options = search_instance.options(self.request)

            # 获取 Select 组件的 Load
            load = search_instance.load(self.request)

            # 根据组件类型创建相应的表单项
            if component == "textField":
                item = field.text(name, label).set_width(None)
            elif component == "selectField":
                item = (
                    field.select(name, label)
                    .set_width(None)
                    .set_options(options)
                    .set_load(load["field"], load["api"])
                )
            elif component == "radioField":
                item = (
                    field.radio(name, label)
                    .set_options(options)
                    .set_option_type("button")
                    .set_button_style("solid")
                )
            elif component == "multipleSelectField":
                item = (
                    field.select(name, label)
                    .set_mode("multiple")
                    .set_width(None)
                    .set_options(options)
                )
            elif component == "dateField":
                item = field.date(name, label).set_width(None)
            elif component == "datetimeField":
                item = field.datetime(name, label).set_width(None)
            elif component == "dateRangeField":
                item = field.date_range(name, label).set_width(None)
            elif component == "datetimeRangeField":
                item = field.datetime_range(name, label).set_width(None)
            elif component == "cascaderField":
                item = (
                    field.cascader(name, label)
                    .set_api(api)
                    .set_width(None)
                    .set_options(options)
                )
            elif component == "treeSelectField":
                item = (
                    field.tree_select(name, label)
                    .set_width(None)
                    .set_tree_data(options)
                )

            search = search.set_items(item)

        return search



from typing import Any

from quark import Request
from quark.template.action import Drawer


class DetailDrawer(Drawer):

    def __init__(self, name: str, detail_component: Any):
        self.name = name
        self.detail_component = detail_component
        self.type = "link"
        self.size = "small"
        self.destroy_on_close = True
        self.reload = "table"
        self.width = 750
        self.set_only_on_index_table_row(True)

    def init(self, request: Request) -> Any:
        # 这里通常 init 方法设置按钮样式等，已在构造函数完成
        return self

    async def get_body(self, request: Request) -> Any:
        return self.detail_component



from tortoise import fields
from tortoise.models import Model


class ActionLog(Model):
    id = fields.IntField(pk=True)
    uid = fields.IntField()
    username = fields.CharField(max_length=20, null=True)
    url = fields.CharField(max_length=500)
    remark = fields.CharField(max_length=255)
    ip = fields.CharField(max_length=100)
    type = fields.CharField(max_length=100)
    status = fields.IntField(default=1)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True, null=True)

    class Meta(Model.Meta):
        table = "action_logs"
        table_description = "操作日志表"



from pydantic import Field, model_validator
from typing import Any, Dict, Optional
from ..component import Component

class Statistic(Component):
    component: str = "statistic"
    decimal_separator: str = Field(".", description="小数点")
    group_separator: str = Field(",", description="千分位标识符")
    precision: int = Field(0, description="数值精度")
    prefix: str = Field("", description="数值的前缀")
    suffix: str = Field("", description="数值的后缀")
    title: str = Field("", description="标题")
    value: int = Field(0, description="数值内容")
    value_style: Dict[str, str] = Field({}, description="数值的样式")
    style: Optional[Dict[str, Any]] = Field(None, description="样式")

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_decimal_separator(self, decimal_separator: str):
        # 设置小数点
        self.decimal_separator = decimal_separator
        return self

    def set_group_separator(self, group_separator: str):
        # 设置千分位标识符
        self.group_separator = group_separator
        return self

    def set_precision(self, precision: int):
        # 设置数值精度
        self.precision = precision
        return self

    def set_prefix(self, prefix: str):
        # 设置数值的前缀
        self.prefix = prefix
        return self

    def set_suffix(self, suffix: str):
        # 设置数值的后缀
        self.suffix = suffix
        return self

    def set_title(self, title: str):
        # 设置标题
        self.title = title
        return self

    def set_value(self, value: int):
        # 设置数值内容
        self.value = value
        return self

    def set_value_style(self, value_style: Dict[str, str]):
        # 设置数值的样式
        self.value_style = value_style
        return self


from pydantic import Field
from ..component import Component
from typing import Any, List, Optional


class Rule(Component):
    """
    表单验证规则
    """

    # 需要验证的字段名称
    name: Optional[str] = Field(default=None, exclude=True)

    # 规则类型，max | min | unique | required
    rule_type: Optional[str] = Field(default=None, exclude=True)

    # 仅在 type 为 array 类型时有效，用于指定数组元素的校验规则
    default_field: Optional[Any] = None

    # 是否匹配枚举中的值（需要将 type 设置为 enum）
    enum: Optional[List[Any]] = None

    # 仅在 type 为 array 或 object 类型时有效，用于指定子元素的校验规则
    fields: Optional[Any] = None

    # string 类型时为字符串长度；number 类型时为确定数字； array 类型时为数组长度
    length: Optional[int] = None

    # 必须设置 type：string 类型为字符串最大长度；number 类型时为最大值；array 类型时为数组最大长度
    max_value: Optional[int] = Field(alias="max", default=None)

    # 错误信息，不设置时会通过模板自动生成
    message: Optional[str] = None

    # 必须设置 type：string 类型为字符串最小长度；number 类型时为最小值；array 类型时为数组最小长度
    min_value: Optional[int] = Field(alias="min", default=None)

    # 正则表达式匹配
    pattern: Optional[str] = None

    # 是否为必选字段
    required_value: Optional[bool] = Field(alias="required", default=None)

    # type：unique时，指定验证的表名
    unique_table: Optional[str] = Field(default=None, exclude=True)

    # type：unique时，指定需验证表中的字段
    unique_table_field: Optional[str] = Field(default=None, exclude=True)

    # type：unique时，忽略符合条件验证的列，例如：{id}
    unique_ignore_value: Optional[str] = Field(default=None, exclude=True)

    # 字段类型，string | number | boolean | method | regexp | integer | float | array | object | enum | date | url | hex | email | any
    rule_type_field: Optional[str] = None

    # 转换前端验证规则，剔除前端不支持的 unique
    @staticmethod
    def convert_to_frontend_rules(rules: List["Rule"]) -> List["Rule"]:
        return [rule for rule in rules if rule.rule_type != "unique"]

    # 必须设置 type：string 类型；为字符串最大长度；number 类型时为最大值；array 类型时为数组最大长度
    @staticmethod
    def max(max_value: int, message: str) -> "Rule":
        rule = Rule()
        return rule.set_max(max_value).set_message(message)

    # 必须设置 type：string 类型为字符串最小长度；number 类型时为最小值；array 类型时为数组最小长度
    @staticmethod
    def min(min_value: int, message: str) -> "Rule":
        rule = Rule()
        return rule.set_min(min_value).set_message(message)

    # 正则表达式匹配
    @staticmethod
    def regexp(pattern: str, message: str) -> "Rule":
        rule = Rule()
        return rule.set_regexp(pattern).set_message(message)

    # 必须为字符串
    @staticmethod
    def string(message: str) -> "Rule":
        rule = Rule()
        return rule.set_string().set_message(message)

    # 必须为数字
    @staticmethod
    def number(message: str) -> "Rule":
        rule = Rule()
        return rule.set_number().set_message(message)

    # 必须为布尔类型
    @staticmethod
    def boolean(message: str) -> "Rule":
        rule = Rule()
        return rule.set_boolean().set_message(message)

    # 必须为整型
    @staticmethod
    def integer(message: str) -> "Rule":
        rule = Rule()
        return rule.set_integer().set_message(message)

    # 必须为浮点型
    @staticmethod
    def float(message: str) -> "Rule":
        rule = Rule()
        return rule.set_float().set_message(message)

    # 必须为邮箱字段
    @staticmethod
    def email(message: str) -> "Rule":
        rule = Rule()
        return rule.set_email().set_message(message)

    # 必须为链接
    @staticmethod
    def url(message: str) -> "Rule":
        rule = Rule()
        return rule.set_url().set_message(message)

    # 必须为手机号
    @staticmethod
    def phone(message: str) -> "Rule":
        rule = Rule()
        return rule.set_phone().set_message(message)

    # 是否为必选字段
    @staticmethod
    def required(message: str) -> "Rule":
        rule = Rule()
        return rule.set_required().set_message(message)

    # 设置 unique 验证类型，插入数据时：Unique("admins", "username", "用户名已存在")，更新数据时：Unique("admins", "username", "{id}", "用户名已存在")
    @staticmethod
    def unique(*unique: str) -> "Rule":
        rule = Rule()
        if len(unique) == 3:
            unique_table, unique_table_field, message = unique
            rule.set_unique(unique_table, unique_table_field)
        elif len(unique) == 4:
            unique_table, unique_table_field, unique_ignore_value, message = unique
            rule.set_unique(unique_table, unique_table_field, unique_ignore_value)
        else:
            raise ValueError("unique方法需要传入3个或4个参数: (table, field, message) 或 (table, field, ignore_value, message)")
        rule.set_message(message)
        return rule

    # 需要验证的字段名称
    def set_name(self, name: str) -> "Rule":
        self.name = name
        return self

    # 必须设置 type：string 类型；为字符串最大长度；number 类型时为最大值；array 类型时为数组最大长度
    def set_max(self, max_value: int) -> "Rule":
        self.max_value = max_value
        return self.set_rule_type("max")

    # 错误信息，不设置时会通过模板自动生成
    def set_message(self, message: str) -> "Rule":
        self.message = message
        return self

    # 必须设置 type：string 类型为字符串最小长度；number 类型时为最小值；array 类型时为数组最小长度
    def set_min(self, min_value: int) -> "Rule":
        self.min_value = min_value
        return self.set_rule_type("min")

    # 正则表达式匹配
    def set_regexp(self, pattern: str) -> "Rule":
        self.pattern = pattern
        return self.set_rule_type("regexp")

    # 是否为必选字段
    def set_required(self) -> "Rule":
        self.required_value = True
        return self.set_rule_type("required")

    # 必须为字符串
    def set_string(self) -> "Rule":
        self.rule_type_field = "string"
        return self.set_rule_type("string")

    # 必须为数字
    def set_number(self) -> "Rule":
        self.rule_type_field = "number"
        return self.set_rule_type("number")

    # 必须为布尔类型
    def set_boolean(self) -> "Rule":
        self.rule_type_field = "boolean"
        return self.set_rule_type("boolean")

    # 必须为整型
    def set_integer(self) -> "Rule":
        self.rule_type_field = "integer"
        return self.set_rule_type("integer")

    # 必须为浮点型
    def set_float(self) -> "Rule":
        self.rule_type_field = "float"
        return self.set_rule_type("float")

    # 必须为邮箱字段
    def set_email(self) -> "Rule":
        self.rule_type_field = "email"
        return self.set_rule_type("email")

    # 必须为链接
    def set_url(self) -> "Rule":
        self.rule_type_field = "url"
        return self.set_rule_type("url")

    # 必须为手机号
    def set_phone(self) -> "Rule":
        return self.set_regexp(r"^1[3-9]\d{9}$")

    # 设置 unique 验证类型，插入数据：SetUnique("admins","username")，更新数据：SetUnique("admins","username","{id}")
    def set_unique(self, *unique: str) -> "Rule":
        self.rule_type_field = "unique"
        if len(unique) == 2:
            self.unique_table, self.unique_table_field = unique
        elif len(unique) == 3:
            self.unique_table, self.unique_table_field, self.unique_ignore_value = (
                unique
            )
        return self.set_rule_type("unique")

    # type：unique时，指定验证的表名
    def set_unique_table(self, unique_table: str) -> "Rule":
        self.unique_table = unique_table
        return self

    # type：unique时，指定验证的表名
    def set_unique_table_field(self, unique_table_field: str) -> "Rule":
        self.unique_table_field = unique_table_field
        return self

    # type：unique时，忽略符合条件验证的列，例如：{id}
    def set_unique_ignore_value(self, unique_ignore_value: str) -> "Rule":
        self.unique_ignore_value = unique_ignore_value
        return self

    # 字段类型，string | number | boolean | url | email
    def set_type(self, rule_type_field: str) -> "Rule":
        self.rule_type_field = rule_type_field
        return self

    # 规则类型，max | min | unique | required
    def set_rule_type(self, rule_type: str) -> "Rule":
        self.rule_type = rule_type
        return self


from pydantic import model_validator, field_validator
from typing import Any, Dict, Optional
from ..component import Component

class Divider(Component):
    component: str = "divider"
    dashed: bool = False
    orientation: str = "center"
    plain: bool = False
    type: str = "horizontal"
    body: Optional[Any] = None

    @field_validator('orientation')
    def validate_orientation(cls, v):
        limits = ["left", "right", "center"]
        if v not in limits:
            raise ValueError("Argument must be in 'left', 'right', 'center'!")
        return v

    @field_validator('type')
    def validate_type(cls, v):
        limits = ["vertical", "horizontal"]
        if v not in limits:
            raise ValueError("Argument must be in 'vertical', 'horizontal'!")
        return v

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self
    # 设置方法（链式调用）
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    def set_dashed(self, dashed: bool):
        self.dashed = dashed
        return self

    def set_orientation(self, orientation: str):
        self.orientation = orientation
        return self

    def set_plain(self, plain: bool):
        self.plain = plain
        return self

    def set_type(self, divider_type: str):
        self.type = divider_type
        return self

    def set_body(self, body: Any):
        self.body = body
        return self


from typing import Dict, List, Optional, Union

from pydantic import model_validator

from ..component import Component
from .item import Item


class Dropdown(Component):
    component: Optional[str] = "dropdown"

    # 按钮文字
    label: Optional[Union[str, List[str]]] = None

    # 是否调整按钮为父元素宽度
    block: Optional[bool] = None

    # 是否为危险按钮
    danger: Optional[bool] = None

    # 按钮是否禁用
    disabled: Optional[bool] = None

    # 幽灵按钮，背景透明
    ghost: Optional[bool] = None

    # 按钮图标
    icon: Optional[Union[str, List[str]]] = None

    # 按钮形状
    shape: Optional[str] = None

    # 按钮大小
    size: Optional[str] = "default"

    # 按钮类型
    type: Optional[str] = "default"

    # 下拉框箭头是否显示
    arrow: Optional[bool] = None

    # 关闭后是否销毁 Dropdown
    destroy_popup_on_hide: Optional[bool] = None

    # 下拉菜单内容
    menu: Optional[Union[List[Item], str]] = None

    # 下拉根元素的类名称
    overlay_class_name: Optional[str] = None

    # 下拉根元素的样式
    overlay_style: Optional[Dict[str, str]] = None

    # 菜单弹出位置
    placement: Optional[str] = "bottomLeft"

    # 触发下拉的行为
    trigger: Optional[List[str]] = ["click"]

    # 菜单是否显示
    visible: Optional[bool] = None

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    def set_style(self, style: Dict[str, str]) -> "Dropdown":
        """
        设置组件样式

        :param style: 样式字典
        :return: 返回当前组件实例
        """
        self.overlay_style = style
        return self

    def set_label(self, label: Union[str, List[str]]) -> "Dropdown":
        """
        设置按钮文字

        :param label: 按钮文字
        :return: 返回当前组件实例
        """
        self.label = label
        return self

    def set_block(self, block: bool) -> "Dropdown":
        """
        设置按钮宽度调整为其父宽度

        :param block: 是否为块级元素
        :return: 返回当前组件实例
        """
        self.block = block
        return self

    def set_danger(self, danger: bool) -> "Dropdown":
        """
        设置是否为危险按钮

        :param danger: 是否为危险按钮
        :return: 返回当前组件实例
        """
        self.danger = danger
        return self

    def set_disabled(self, disabled: bool) -> "Dropdown":
        """
        设置按钮是否禁用

        :param disabled: 是否禁用
        :return: 返回当前组件实例
        """
        self.disabled = disabled
        return self

    def set_ghost(self, ghost: bool) -> "Dropdown":
        """
        设置幽灵按钮，背景透明

        :param ghost: 是否为幽灵按钮
        :return: 返回当前组件实例
        """
        self.ghost = ghost
        return self

    def set_icon(self, icon: Union[str, List[str]]) -> "Dropdown":
        """
        设置按钮图标

        :param icon: 图标，可以是字符串或字符串列表
        :return: 返回当前组件实例
        """
        if isinstance(icon, str):
            self.icon = f"{icon}"
        elif isinstance(icon, list) and len(icon) == 2:
            self.icon = [f"{icon[0]}", f"{icon[1]}"]
        return self

    def set_shape(self, shape: str) -> "Dropdown":
        """
        设置按钮形状

        :param shape: 按钮形状 (circle | round)
        :return: 返回当前组件实例
        """
        self.shape = shape
        return self

    def set_type(self, button_type: str, danger: bool = False) -> "Dropdown":
        """
        设置按钮类型

        :param button_type: 按钮类型
        :param danger: 是否为危险按钮
        :return: 返回当前组件实例
        """
        self.type = button_type
        self.danger = danger
        return self

    def set_size(self, size: str) -> "Dropdown":
        """
        设置按钮大小

        :param size: 按钮大小 (large | middle | small | default)
        :return: 返回当前组件实例
        """
        self.size = size
        return self

    def set_arrow(self, arrow: bool) -> "Dropdown":
        """
        设置下拉框箭头是否显示

        :param arrow: 是否显示箭头
        :return: 返回当前组件实例
        """
        self.arrow = arrow
        return self

    def set_destroy_popup_on_hide(self, destroy_popup_on_hide: bool) -> "Dropdown":
        """
        设置关闭后是否销毁 Dropdown

        :param destroy_popup_on_hide: 是否销毁
        :return: 返回当前组件实例
        """
        self.destroy_popup_on_hide = destroy_popup_on_hide
        return self

    def set_menu(self, menu: Union[List[Item], str]) -> "Dropdown":
        """
        设置菜单内容

        :param menu: 菜单，可以是列表或字符串
        :return: 返回当前组件实例
        """
        self.menu = menu
        return self

    def set_overlay_class_name(self, overlay_class_name: str) -> "Dropdown":
        """
        设置下拉根元素的类名称

        :param overlay_class_name: 类名称
        :return: 返回当前组件实例
        """
        self.overlay_class_name = overlay_class_name
        return self

    def set_overlay_style(self, overlay_style: Dict[str, str]) -> "Dropdown":
        """
        设置下拉根元素的样式

        :param overlay_style: 样式字典
        :return: 返回当前组件实例
        """
        self.overlay_style = overlay_style
        return self

    def set_placement(self, placement: str) -> "Dropdown":
        """
        设置菜单弹出位置

        :param placement: 菜单弹出位置
        :return: 返回当前组件实例
        """
        self.placement = placement
        return self

    def set_trigger(self, trigger: List[str]) -> "Dropdown":
        """
        设置触发下拉的行为

        :param trigger: 触发行为 (click | hover | contextMenu)
        :return: 返回当前组件实例
        """
        self.trigger = trigger
        return self

    def set_visible(self, visible: bool) -> "Dropdown":
        """
        设置菜单是否显示

        :param visible: 是否显示菜单
        :return: 返回当前组件实例
        """
        self.visible = visible
        return self



import re
from typing import Any, Dict, List, Optional, Type
from urllib.parse import parse_qs, urlparse

from fastapi import Request
from tortoise import Tortoise
from tortoise.expressions import Q
from tortoise.models import Model

from ..component.form import Rule
from ..component.form.fields.when import Item, When


class PerformsValidation:

    # 请求对象
    request: Request = None

    # 字段
    fields: Optional[Any] = None

    def __init__(
        self,
        request: Optional[Request] = None,
        fields: Optional[Any] = None,
    ):
        self.request = request
        self.fields = fields

    async def validator_for_creation(self, data: Dict[str, Any]):
        rules = await self.rules_for_creation()
        validator = await self.validator(rules, data)
        if validator:
            raise ValueError(validator)

    async def validator_for_update(self, data: Dict[str, Any]):
        rules = await self.rules_for_update()
        validator = await self.validator(rules, data)
        if validator:
            raise ValueError(validator)

    async def validator_for_import(self, data: Dict[str, Any]):
        rules = await self.rules_for_import()
        validator = await self.validator(rules, data)
        if validator:
            raise ValueError(validator)

    async def validator(self, rules: List[Rule], data: Dict[str, Any]) -> Optional[str]:
        for rule in rules:
            field_value = data.get(rule.name)
            if rule.rule_type == "required":
                if field_value is None or field_value == "":
                    return rule.message
            elif rule.rule_type == "min":
                if isinstance(field_value, str):
                    if len(field_value) < rule.min_value:
                        return rule.message
            elif rule.rule_type == "max":
                if isinstance(field_value, str):
                    if len(field_value) > rule.max_value:
                        return rule.message
            elif rule.rule_type == "regexp":
                if isinstance(field_value, str):
                    # 标准化正则表达式模式
                    pattern = rule.pattern
                    if pattern.startswith("/") and pattern.endswith("/"):
                        pattern = pattern[1:-1]
                    if not re.fullmatch(pattern, field_value):
                        return rule.message
            elif rule.rule_type == "unique":
                try:
                    # 通过 Tortoise ORM 获取模型
                    table: Type[Model] = Tortoise.apps.get("models").get(
                        rule.unique_table
                    )
                    if not table:
                        # 如果通过表名找不到，尝试通过类名查找
                        table = Tortoise.apps.get("models").get(
                            rule.unique_table.rstrip("s").capitalize()
                        )

                    if not table or not issubclass(table, Model):
                        return f"验证规则错误: 找不到模型 {rule.unique_table}"

                    # 其余代码保持不变
                    field = rule.unique_table_field
                    ignore_value = rule.unique_ignore_value

                    # 构建查询条件
                    filter_kwargs = {f"{field}": field_value}
                    query = table.filter(**filter_kwargs)

                    # 处理忽略值（用于更新时排除自身）
                    if (
                        ignore_value
                        and ignore_value.startswith("{")
                        and ignore_value.endswith("}")
                    ):
                        ignore_key = ignore_value.strip("{}")
                        ignore_val = data.get(ignore_key)
                        if ignore_val is not None:
                            query = query.filter(~Q(**{f"{ignore_key}": ignore_val}))

                    count = await query.count()
                    if count > 0:
                        return rule.message

                except KeyError:
                    return f"验证规则错误: 找不到模型 {rule.unique_table}"
                except Exception as e:
                    return f"验证规则错误: {str(e)}"
        return None

    async def rules_for_creation(self) -> List[Rule]:
        rules = []
        for v in self.fields:
            rules.extend(await self.get_rules_for_creation(v))
            if isinstance(v, When):
                when_component = v
                if when_component.items:
                    for vi in when_component.items:
                        if await self.need_validate_when_rules(vi):
                            body = vi.body
                            if body:
                                if isinstance(body, list):
                                    for bv in body:
                                        rules.extend(
                                            await self.get_rules_for_creation(bv)
                                        )
                                else:
                                    rules.extend(
                                        await self.get_rules_for_creation(body)
                                    )
        return rules

    async def rules_for_update(self) -> List[Rule]:
        rules = []
        for v in self.fields:
            rules.extend(await self.get_rules_for_update(v))
            if isinstance(v, When):
                when_component = v
                if when_component.items:
                    for vi in when_component.items:
                        if await self.need_validate_when_rules(vi):
                            body = vi.body
                            if body:
                                if isinstance(body, list):
                                    for bv in body:
                                        rules.extend(
                                            await self.get_rules_for_update(bv)
                                        )
                                else:
                                    rules.extend(await self.get_rules_for_update(body))
        return rules

    async def rules_for_import(self) -> List[Rule]:
        rules = []
        for v in self.fields:
            rules.extend(await self.get_rules_for_creation(v))
            if isinstance(v, When):
                when_component = v
                if when_component.items:
                    for vi in when_component.items:
                        if await self.need_validate_when_rules(vi):
                            body = vi.body
                            if body:
                                if isinstance(body, list):
                                    for bv in body:
                                        rules.extend(
                                            await self.get_rules_for_creation(bv)
                                        )
                                else:
                                    rules.extend(
                                        await self.get_rules_for_creation(body)
                                    )
        return rules

    async def need_validate_when_rules(self, when_item: Item) -> bool:
        cond_name = when_item.condition_name
        cond_opt = when_item.option
        cond_op = when_item.condition_operator
        parsed = urlparse(self.request.url.path)
        query_params = parse_qs(parsed.query)
        value = query_params.get(cond_name, [""])[0]

        if not value:
            return False

        if cond_op == "=":
            return value == cond_opt
        elif cond_op == ">":
            return value > cond_opt
        elif cond_op == "<":
            return value < cond_opt
        elif cond_op == "<=":
            return value <= cond_opt
        elif cond_op == ">=":
            return value >= cond_opt
        elif cond_op == "has":
            if isinstance(cond_opt, list):
                return value in cond_opt
            else:
                return cond_opt in value
        elif cond_op == "in":
            if isinstance(cond_opt, list):
                return value in cond_opt
            else:
                return value == cond_opt
        else:
            return value == cond_opt

    async def get_rules_for_creation(self, field: Any) -> List[Rule]:
        rules = []
        if hasattr(field, "get_rules"):
            rules.extend(field.get_rules())
        if hasattr(field, "get_creation_rules"):
            rules.extend(field.get_creation_rules())
        return rules

    async def get_rules_for_update(self, field: Any) -> List[Rule]:
        rules = []
        if hasattr(field, "get_rules"):
            rules.extend(field.get_rules())
        if hasattr(field, "get_update_rules"):
            rules.extend(field.get_update_rules())
        return rules



from pydantic import model_validator
from typing import Any, Dict, List, Optional
from ..component import Component

class Layout(Component):
    component: str = "layout"
    cache: bool = True
    title: Optional[str] = None
    logo: Optional[Any] = None
    loading: bool = False
    content_style: Optional[Dict[str, str]] = None
    actions: Optional[Any] = None
    layout: Optional[str] = None
    split_menus: bool = None
    content_width: Optional[str] = None
    primary_color: Optional[str] = None
    fixed_header: bool = None
    fix_siderbar: bool = None
    iconfont_url: Optional[str] = None
    locale: Optional[str] = None
    sider_width: Optional[int] = None
    menu: Optional[Any] = None
    right_menus: Optional[List[Any]] = None  # 右上角菜单
    footer: Optional[Any] = None
    body: Optional[Any] = None

    @model_validator(mode="after")
    def init(self):
        self.cache = True
        self.set_key()
        return self

    # Set style.
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    # 是否缓存layout
    def set_cache(self, cache: bool):
        self.cache = cache
        return self

    # layout 的左上角 的 title
    def set_title(self, title: str):
        self.title = title
        return self

    # layout 的左上角的 logo
    def set_logo(self, logo: Any):
        self.logo = logo
        return self

    # layout 的加载态
    def set_loading(self, loading: bool):
        self.loading = loading
        return self

    # layout 的内容区 style
    def set_content_style(self, content_style: Dict[str, str]):
        self.content_style = content_style
        return self

    # layout 的头部行为
    def set_actions(self, actions: Any):
        self.actions = actions
        return self

    # layout 的布局模式，side：右侧导航，top：顶部导航，mix：混合模式
    def set_layout(self, layout: str):
        self.layout = layout
        return self

    # layout 的内容模式,Fluid：定宽 1200px，Fixed：自适应
    def set_content_width(self, content_width: str):
        self.content_width = content_width
        return self

    # 容器控件里面的内容
    def set_primary_color(self, primary_color: str):
        self.primary_color = primary_color
        return self

    # 是否固定 header 到顶部
    def set_fixed_header(self, fixed_header: bool):
        self.fixed_header = fixed_header
        return self

    # 是否固定导航
    def set_fix_siderbar(self, fix_siderbar: bool):
        self.fix_siderbar = fix_siderbar
        return self

    # 使用 IconFont 的图标配置
    def set_iconfont_url(self, iconfont_url: str):
        self.iconfont_url = iconfont_url
        return self

    # 当前 layout 的语言设置，'zh-CN' | 'zh-TW' | 'en-US'
    def set_locale(self, locale: str):
        self.locale = locale
        return self

    # 侧边菜单宽度
    def set_sider_width(self, sider_width: int):
        self.sider_width = sider_width
        return self

    # 自动分割菜单
    def set_split_menus(self, split_menus: bool):
        self.split_menus = split_menus
        return self

    # 菜单
    def set_menu(self, menu: Any):
        self.menu = menu
        return self

    # 右上角菜单
    def set_right_menus(self, right_menus: List[Any]):
        self.right_menus = right_menus
        return self

    # 页脚
    def set_footer(self, footer: Any):
        self.footer = footer
        return self

    # 内容
    def set_body(self, body: Any):
        self.body = body
        return self


from .action_log import ActionLog
from .attachment import Attachment
from .attachment_category import AttachmentCategory
from .config import Config
from .department import Department
from .menu import Menu
from .menu_permission import MenuPermission
from .permission import Permission
from .position import Position
from .role import Role
from .role_department import RoleDepartment
from .role_menu import RoleMenu
from .role_permission import RolePermission
from .user import User
from .user_role import UserRole



from quark import Request

from ...component.form.fields.select import Option
from .search import Search


class Select(Search):
    """
    下拉框
    """

    component = "selectField"

    def __init__(self, column: str = "", name: str = ""):
        self.name = name
        self.column = column
        return self

    # 设置 Option
    def option(self, label: str, value):
        return Option(label=label, value=value)

    # 默认选项列表（子类可重写）
    def options(self, request: Request):
        return []

    # 单向联动，返回数据类型：{"field": "you_want_load_field", "api": "admin/resource_name/action/select-options"}
    def load(self, request: Request):
        return {"field": "", "api": ""}

    # 设置属性，示例：
    # [{"value": 1, "label": "新闻"}, {"value": 2, "label": "音乐"}, {"value": 3, "label": "体育"}]
    #
    # 或者
    #
    # set_options(options, "label_name", "value_name")
    def set_options(self, *args):
        if len(args) == 1 and isinstance(args[0], list):
            self.select_options = args[0]
            return self

        if len(args) == 3:
            # 假设 list_to_options 是一个用于从列表中提取 label 和 value 的方法
            self.select_options = self.list_to_options(args[0], args[1], args[2])

        return self

    @staticmethod
    def list_to_options(data, label_key, value_key):
        """模拟从列表结构生成选项"""
        return [{"label": item[label_key], "value": item[value_key]} for item in data]



from quark.template.action import Action


class FormReset(Action):

    def __init__(self):

        # 文字
        self.name = "重置"

        # 类型
        self.type = "default"

        # 行为类型
        self.action_type = "reset"

        # 设置在表单页展示
        self.set_show_on_form()



from quark import APIRouter, JSONResponse, Request, jsonable_encoder

from .. import loader

router = APIRouter(prefix="/api/admin", tags=["后台增、删、改、查"])


# 列表
@router.get("/{resource}/index")
async def index(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.index_render(request), exclude_none=True)
    )


# 表格行内编辑
@router.get("/{resource}/editable")
async def editable(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.editable_render(request), exclude_none=True)
    )


# 执行行为
@router.get("/{resource}/action/{uriKey}")
async def action_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.action_render(request), exclude_none=True)
    )


# 行为表单值
@router.get("/{resource}/action/{uriKey}/values")
async def action_values_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(
            await res.action_values_render(request), exclude_none=True
        )
    )


# 创建页面
@router.get("/{resource}/create")
async def creation_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.creation_render(request), exclude_none=True)
    )


# 创建方法
@router.post("/{resource}/store")
async def store_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.store_render(request), exclude_none=True)
    )


# 编辑页面
@router.get("/{resource}/edit")
async def edit_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.edit_render(request), exclude_none=True)
    )


# 获取编辑表单值
@router.get("/{resource}/edit/values")
async def edit_values_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(
            await res.edit_values_render(request), exclude_none=True
        )
    )


# 保存编辑值
@router.post("/{resource}/save")
async def save_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.save_render(request), exclude_none=True)
    )


# 导入数据
@router.post("/{resource}/import")
async def import_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.import_render(request), exclude_none=True)
    )


# 导出数据
@router.get("/{resource}/export")
async def export_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.export_render(request), exclude_none=True)
    )


# 详情页
@router.get("/{resource}/detail")
async def detail_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.detail_render(request), exclude_none=True)
    )


# 获取详情页值
@router.get("/{resource}/detail/values")
async def detail_values_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(
            await res.detail_values_render(request), exclude_none=True
        )
    )


# 导入模板
@router.get("/{resource}/import/template")
async def import_template_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(
            await res.import_template_render(request), exclude_none=True
        )
    )


# 表单页
@router.get("/{resource}/form")
async def form_render(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Resource")
    return JSONResponse(
        content=jsonable_encoder(await res.form_render(request), exclude_none=True)
    )



from typing import Any, List

from quark import Request
from quark.component.action.action import Action
from quark.component.form.form import Form
from quark.template.action import Drawer
from quark.template.resolves_fields import ResolvesFields


class MenuEditDrawer(Drawer):

    def __init__(self, resource: Any):
        self.resource = resource
        self.name = "编辑"
        self.type = "link"
        self.size = "small"
        self.destroy_on_close = True
        self.reload = "table"
        self.width = 750
        self.set_only_on_index_table_row(True)

    async def get_body(self, request: Request) -> Any:
        api = await self.resource.update_api(request)
        init_api = await self.resource.edit_value_api(request)
        fields = ResolvesFields(
            request=request,
            fields=await self.resource.fields(request),
        ).update_fields_within_components()

        return (
            Form()
            .set_api(api)
            .set_init_api(init_api)
            .set_body(fields)
            .set_layout("vertical")
            .set_key("editDrawerForm", False)
        )

    async def get_actions(self, request: Request) -> List[Any]:
        return [
            Action().set_label("取消").set_action_type("cancel"),
            Action()
            .set_submit_form("editDrawerForm")
            .set_label("提交")
            .set_with_loading(True)
            .set_reload("table")
            .set_action_type("submit")
            .set_type("primary", False),
        ]



from pydantic import Field, model_validator
from typing import Any, Dict, Optional, Union
from ..component import Component

class Col(Component):
    component: str = "col"
    flex: str = Field("", description="布局属性")
    offset: int = Field(0, description="栅格左侧的间隔格数，间隔内不可以有栅格")
    order: int = Field(0, description="栅格顺序")
    pull: int = Field(0, description="栅格向左移动格数")
    push: int = Field(0, description="栅格向右移动格数")
    span: int = Field(0, description="栅格占位格数，为 0 时相当于 display: none")
    xs: Optional[Union[int, Dict[str, Any]]] = Field(None, description="屏幕 < 576px 响应式栅格，可为栅格数或一个包含其他属性的对象")
    sm: Optional[Union[int, Dict[str, Any]]] = Field(None, description="屏幕 ≥ 576px 响应式栅格，可为栅格数或一个包含其他属性的对象")
    md: Optional[Union[int, Dict[str, Any]]] = Field(None, description="屏幕 ≥ 768px 响应式栅格，可为栅格数或一个包含其他属性的对象")
    lg: Optional[Union[int, Dict[str, Any]]] = Field(None, description="屏幕 ≥ 992px 响应式栅格，可为栅格数或一个包含其他属性的对象")
    xl: Optional[Union[int, Dict[str, Any]]] = Field(None, description="屏幕 ≥ 1200px 响应式栅格，可为栅格数或一个包含其他属性的对象")
    xxl: Optional[Union[int, Dict[str, Any]]] = Field(None, description="屏幕 ≥ 1600px 响应式栅格，可为栅格数或一个包含其他属性的对象")
    body: Optional[Any] = Field(None, description="卡牌内容")
    style: Optional[Dict[str, Any]] = Field(None, description="样式")

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_flex(self, flex: str):
        # 设置布局属性
        self.flex = flex
        return self

    def set_offset(self, offset: int):
        # 设置栅格左侧的间隔格数
        self.offset = offset
        return self

    def set_order(self, order: int):
        # 设置栅格顺序
        self.order = order
        return self

    def set_pull(self, pull: int):
        # 设置栅格向左移动格数
        self.pull = pull
        return self

    def set_push(self, push: int):
        # 设置栅格向右移动格数
        self.push = push
        return self

    def set_span(self, span: int):
        # 设置栅格占位格数
        self.span = span
        return self

    def set_xs(self, xs: Union[int, Dict[str, Any]]):
        # 设置屏幕 < 576px 响应式栅格
        self.xs = xs
        return self

    def set_sm(self, sm: Union[int, Dict[str, Any]]):
        # 设置屏幕 ≥ 576px 响应式栅格
        self.sm = sm
        return self

    def set_md(self, md: Union[int, Dict[str, Any]]):
        # 设置屏幕 ≥ 768px 响应式栅格
        self.md = md
        return self

    def set_lg(self, lg: Union[int, Dict[str, Any]]):
        # 设置屏幕 ≥ 992px 响应式栅格
        self.lg = lg
        return self

    def set_xl(self, xl: Union[int, Dict[str, Any]]):
        # 设置屏幕 ≥ 1200px 响应式栅格
        self.xl = xl
        return self

    def set_xxl(self, xxl: Union[int, Dict[str, Any]]):
        # 设置屏幕 ≥ 1600px 响应式栅格
        self.xxl = xxl
        return self

    def set_body(self, body: Any):
        # 设置卡牌内容
        self.body = body
        return self


from typing import Optional, Any
from .base import Base


class Space(Base):

    component: str = "spaceField"
    """
    组件名称
    """

    align: str = ""
    """
    对齐方式，start | end | center | baseline，默认值为 ""
    """

    direction: str = ""
    """
    间距方向，默认值为 ""
    """

    size: str = ""
    """
    间距大小，默认值为 ""
    """

    split: str = ""
    """
    设置拆分，默认值为 ""
    """

    wrap: bool = False
    """
    是否自动换行，仅在 horizontal 时有效，默认值为 False
    """

    body: Optional[Any] = None
    """
    组件内容，默认值为 None
    """

    def set_align(self, align: str):
        """
        设置对齐方式。

        Args:
            align (str): 对齐方式，如 'start', 'end', 'center', 'baseline'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.align = align
        return self

    def set_direction(self, direction: str):
        """
        设置间距方向。

        Args:
            direction (str): 间距方向。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.direction = direction
        return self

    def set_size(self, size: str):
        """
        设置间距大小。

        Args:
            size (str): 间距大小，如 'small', 'middle', 'large' 或数字。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_split(self, split: str):
        """
        设置拆分卡片的方向。

        Args:
            split (str): 拆分方向，如 'vertical', 'horizontal'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.split = split
        return self

    def set_wrap(self, wrap: bool):
        """
        设置是否自动换行。

        Args:
            wrap (bool): 是否自动换行。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.wrap = wrap
        return self

    def set_body(self, body: Any):
        """
        设置容器控件里面的内容。

        Args:
            body (Any): 组件内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.body = body
        return self



from typing import Any, List

from quark import Request, Resource, models,services
from quark.app import actions, searches
from quark.component.form import field

class ActionLog(Resource):
    """
    操作日志管理
    """

    async def init(self, request: Request):

        # 页面标题
        self.title = "日志"

        # 模型
        self.model = models.ActionLog

        return self

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        async def get_username(row) -> str:
            user_info = await services.UserService().get_info_by_id(row.uid)
            return "账号：<a href='#/layout/index?api=/api/admin/user/detail&id=" + str(user_info.id) + "'>" + user_info.username + "</a><br/>昵称：" + user_info.nickname
        
        return [
            field.id("id", "ID"),
            field.text("username", "用户信息", get_username),
            field.text("url", "行为"),
            field.text("ip", "IP").set_ellipsis(True),
            field.datetime("created_at", "发生时间"),
        ]

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("username", "账号"),
            searches.Input("url", "行为"),
            searches.Input("ip", "IP"),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.BatchDelete(),
            actions.Delete(),
        ]



from typing import Any, Dict, Optional

from .base import Base


class Option(Base):
    """
    表示 Switch 组件选项的类，包含选中和未选中时的内容。

    Attributes:
        checked_children (Any): 选中时的内容。
        un_checked_children (Any): 未选中时的内容。
    """

    checked_children: Any = None
    un_checked_children: Any = None


class Switch(Base):

    component: Optional[str] = "switchField"
    """
    组件名称
    """

    auto_focus: Optional[bool] = None
    """
    默认获取焦点，默认值为 False
    """

    checked: Optional[bool] = None
    """
    指定当前是否选中，默认值为 False
    """

    checked_children: Optional[Any] = None
    """
    选中时的内容
    """

    class_name: Optional[str] = None
    """
    Switch 器类名
    """

    default_checked: Optional[bool] = None
    """
    初始是否选中，默认值为 False
    """

    default_value: Optional[Any] = None
    """
    默认选中的选项
    """

    disabled: Optional[bool] = None
    """
    整组失效，默认值为 False
    """

    loading: Optional[bool] = None
    """
    加载中状态，默认值为 False
    """

    size: Optional[str] = None
    """
    选择框大小
    """

    un_checked_children: Optional[Any] = None
    """
    自定义的选择框后缀图标
    """

    value: Optional[Any] = None
    """
    值
    """

    def get_options(self) -> Dict[int, Any]:
        """
        获取当前可选项。

        Returns:
            Dict[int, Any]: 可选项字典。
        """
        data: Dict[int, Any] = {0: self.un_checked_children, 1: self.checked_children}
        return data

    def set_options(self, options: Option):
        """
        设置选项。

        Args:
            options (Option): 选项对象。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.checked_children = options.checked_children
        self.un_checked_children = options.un_checked_children
        return self

    def set_auto_focus(self, auto_focus: bool):
        """
        设置默认获取焦点。

        Args:
            auto_focus (bool): 是否默认获取焦点。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.auto_focus = auto_focus
        return self

    def set_checked(self, checked: bool):
        """
        指定当前是否选中。

        Args:
            checked (bool): 是否选中。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.checked = checked
        return self

    def set_checked_children(self, checked_children: Any):
        """
        设置选中时的内容。

        Args:
            checked_children (Any): 选中时的内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.checked_children = checked_children
        return self

    def set_class_name(self, class_name: str):
        """
        设置 Switch 器类名。

        Args:
            class_name (str): 类名。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.class_name = class_name
        return self

    def set_default_checked(self, default_checked: bool):
        """
        设置初始是否选中。

        Args:
            default_checked (bool): 初始是否选中。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.default_checked = default_checked
        return self

    def set_loading(self, loading: bool):
        """
        设置加载中状态。

        Args:
            loading (bool): 是否加载中。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.loading = loading
        return self

    def set_size(self, size: str):
        """
        设置选择框大小。

        Args:
            size (str): 选择框大小。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_un_checked_children(self, un_checked_children: Any):
        """
        设置非选中时的内容。

        Args:
            un_checked_children (Any): 非选中时的内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.un_checked_children = un_checked_children
        return self

    def set_true_value(self, value: Any):
        """
        设置选中时的内容。

        Args:
            value (Any): 选中时的内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.checked_children = value
        return self

    def set_false_value(self, value: Any):
        """
        设置非选中时的内容。

        Args:
            value (Any): 非选中时的内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.un_checked_children = value
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def get_value_enum(self) -> Dict[int, Any]:
        """
        获取当前列值的枚举 value_enum。

        Returns:
            Dict[int, Any]: 列值枚举字典。
        """
        data: Dict[int, Any] = {0: self.un_checked_children, 1: self.checked_children}
        return data

    def get_option_label(self, value: Any) -> Any:
        """
        根据 value 值获取 Option 的 Label。

        Args:
            value (Any): 值。

        Returns:
            Any: Option 的 Label。
        """
        if value == 1:
            return self.checked_children
        else:
            return self.un_checked_children

    def get_option_value(self, label: str) -> bool:
        """
        根据 label 值获取 Option 的 Value。

        Args:
            label (str): 标签。

        Returns:
            bool: Option 的 Value。
        """
        return self.checked_children == label

    def get_option_labels(self) -> str:
        """
        获取 Option 的 Labels。

        Returns:
            str: Option 的 Labels 字符串。
        """
        label_string = ""
        if isinstance(self.checked_children, str) and isinstance(
            self.un_checked_children, str
        ):
            label_string = f"{self.checked_children},{self.un_checked_children}"
        return label_string.strip(",")



from quark.template.action import Action


class FormSubmit(Action):

    def __init__(self):

        # 文字
        self.name = "提交"

        # 类型（按钮颜色/样式）
        self.type = "primary"

        # 行为类型：submit 表单提交
        self.action_type = "submit"

        # 是否启用 loading 效果（仅 ajax、submit 类型支持）
        self.with_loading = True

        # 设置仅在表单中展示
        self.set_only_on_form(True)



from pydantic import model_validator
from ..component import Component


class Divider(Component):
    component: str = "menuDivider"
    dashed: bool = False

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    # 子菜单项值
    def set_dashed(self, dashed: bool):
        self.dashed = dashed
        return self



import json
from typing import Any, Dict, List

from tortoise import Model

from quark import Message, Request, Resource, models
from quark.app import actions
from quark.component.form import field
from quark.component.tabs.tab_pane import TabPane
from quark.models.config import Config
from quark.services.config import ConfigService


class WebConfig(Resource):
    """
    网站配置
    """

    async def init(self, request: Request) -> Any:

        # 页面标题
        self.title = "网站配置"

        # 模型
        self.model = models.Config

        return self

    async def fields(self, request: Request) -> List[Any]:
        # 获取所有分组
        group_names = (
            await Config.filter(status=1)
            .distinct()
            .values_list("group_name", flat=True)
        )

        tab_panes = []
        for group_name in group_names:
            configs = (
                await Config.filter(status=1, group_name=group_name)
                .order_by("sort")
                .values()
            )
            fields_list = []

            for config in configs:
                remark = config.get("remark", "")
                if config["type"] == "text":
                    f = field.text(config["name"], config["title"]).set_extra(remark)
                elif config["type"] == "textarea":
                    f = field.textarea(config["name"], config["title"]).set_extra(
                        remark
                    )
                elif config["type"] == "file":
                    f = (
                        field.file(config["name"], config["title"])
                        .set_extra(remark)
                        .set_button(f"上传{config['title']}")
                    )
                elif config["type"] == "picture":
                    f = (
                        field.image(config["name"], config["title"])
                        .set_extra(remark)
                        .set_button(f"上传{config['title']}")
                    )
                elif config["type"] == "switch":
                    f = (
                        field.switch(config["name"], config["title"])
                        .set_editable(True)
                        .set_true_value("正常")
                        .set_false_value("禁用")
                        .set_default_value(True)
                        .set_extra(remark)
                    )
                else:
                    continue
                fields_list.append(f)

            tab_panes.append(TabPane(title=str(group_name), body=fields_list))

        return tab_panes

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.FormSubmit(),
            actions.FormReset(),
            actions.FormBack(),
            actions.FormExtraBack(),
        ]

    async def before_form_showing(self, request: Request) -> Any:
        configs = await Config.filter(status=1).values()
        data = {}
        for config in configs:
            value = config["value"]
            if config["type"] == "switch":
                data[config["name"]] = value != "0"
            elif config["type"] in ["picture", "file"]:
                if value:
                    try:
                        json_data = json.loads(value)
                        data[config["name"]] = json_data
                    except:
                        data[config["name"]] = value
                else:
                    data[config["name"]] = None
            else:
                data[config["name"]] = value
        return data

    async def form_handle(
        self, request: Request, model: Model, data: Dict[str, Any]
    ) -> Any:
        try:
            for k, v in data.items():
                if isinstance(v, (list, dict)):
                    v = json.dumps(v)
                await Config.filter(name=k).update(value=v)
        except Exception as e:
            return Message.error(str(e))

        # 刷新网站配置
        await ConfigService().refresh()

        return Message.success("操作成功")



from typing import Any, Optional
from .base import Base


class List(Base):

    component: str = "listField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认的选中项
    """

    disabled: bool = False
    """
    整组失效，默认 False
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    items: Optional[Any] = None
    """
    列表项
    """

    button_text: str = "添加一行数据"
    """
    按钮文字，默认 "添加一行数据"
    """

    button_position: str = "top"
    """
    按钮位置，默认 "top"
    """

    always_show_item_label: bool = True
    """
    Item 中总是展示 label，默认 True
    """

    def set_button(self, text: str, position: str):
        """
        设置按钮文字和位置。

        Args:
            text (str): 按钮文字。
            position (str): 按钮位置。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.button_text = text
        self.button_position = position
        return self

    def set_item(self, callback: Any):
        """
        设置表单项。

        Args:
            callback (Any): 回调函数。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        if callable(callback):
            self.items = callback()
        return self

    def set_always_show_item_label(self, always_show_item_label: bool):
        """
        设置 Item 中是否总是展示 label。

        Args:
            always_show_item_label (bool): 是否总是展示 label。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.always_show_item_label = always_show_item_label
        return self

    def set_api(self, api: str):
        """
        设置上传的 api 接口。

        Args:
            api (str): api 接口地址。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.api = api
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from pydantic import BaseModel
from typing import Any, Optional
from ..component import Component

class Lists(Component):
    row_key: str = ""
    api: str = ""
    api_type: str = ""
    header_title: str = ""
    metas: Optional[Any] = None
    row_selection: Optional[Any] = None
    striped: bool = False
    datasource: Optional[Any] = None
    pagination: Optional[Any] = None
    tool_bar: Optional[Any] = None
    style: Optional[dict[str, Any]] = None

    # Set style.
    def set_style(self, style: dict[str, Any]) -> 'Lists':
        self.style = style
        return self

    # layout 的左上角 的 title
    def set_row_key(self, row_key: str) -> 'Lists':
        self.row_key = row_key
        return self

    # 获取表格数据接口
    def set_api(self, api: str) -> 'Lists':
        self.api = api
        return self

    # 获取表格数据接口类型
    def set_api_type(self, api_type: str) -> 'Lists':
        self.api_type = api_type
        return self

    # 表头标题
    def set_title(self, title: str) -> 'Lists':
        self.header_title = title
        return self

    # 表头标题
    def set_header_title(self, header_title: str) -> 'Lists':
        self.header_title = header_title
        return self

    # 批量设置表格列
    def set_metas(self, metas: Any) -> 'Lists':
        limits = [
            "type",
            "title",
            "subTitle",
            "description",
            "avatar",
            "actions",
            "content",
            "extra"
        ]

        if isinstance(metas, dict):
            for k in metas.keys():
                if k not in limits:
                    raise ValueError("meta index key must be in 'type','title','subTitle','description','avatar','actions','content','extra'!")
        self.metas = metas
        return self

    # 批量操作选择项
    def set_row_selection(self, row_selection: list[Any]) -> 'Lists':
        self.row_selection = row_selection
        return self

    # 透传 ProUtils 中的 ListToolBar 配置项
    def set_tool_bar(self, tool_bar: Any) -> 'Lists':
        self.tool_bar = tool_bar
        return self

    # 设置表格滚动
    def set_striped(self, striped: bool) -> 'Lists':
        self.striped = striped
        return self

    # 表格数据
    def set_datasource(self, datasource: Any) -> 'Lists':
        self.datasource = datasource
        return self

    # 表格分页
    def set_pagination(self, current: int, page_size: int, total: int, default_current: int) -> 'Lists':
        self.pagination = {
            "current": current,
            "page_size": page_size,
            "total": total,
            "default_current": default_current
        }
        return self


from typing import Any, List, Optional
from ...component import Component
from .base import Base
import re


class Option(Component):
    label: str  # 选项显示文本
    value: Any  # 选项值
    disabled: bool = False  # 是否禁用


class Checkbox(Base):

    component: str = "checkboxField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认值
    """

    disabled: bool = False
    """
    是否禁用
    """

    value: Optional[Any] = None
    """
    当前值
    """

    options: List[Option] = []
    """
    选项列表
    """

    def get_options(self) -> List[Option]:
        """
        获取当前可选项。

        Returns:
            List[Option]: 可选项列表。
        """
        return self.options

    def build_options(
        self, items: List[Any], label_name: str, value_name: str
    ) -> List[Option]:
        """通过反射构建选项列表"""
        options = []
        for item in items:
            try:
                value = getattr(item, value_name)
                label = getattr(item, label_name)
            except AttributeError:
                continue
            options.append(Option(label=label, value=value))
        return options

    def list_to_options(
        self, items: List[Any], label_name: str, value_name: str
    ) -> List[Option]:
        """从列表中构建选项"""
        return self.build_options(items, label_name, value_name)

    def set_options(self, *args):
        """设置选项"""
        if len(args) == 1 and isinstance(args[0], list):
            self.options = args[0]
        elif len(args) == 3:
            self.options = self.list_to_options(*args)
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def get_option_label(self, value: Any) -> str:
        """根据值获取标签"""
        labels = [opt.label for opt in self.options if opt.value == value]
        return ", ".join(labels)

    def get_option_value(self, label: str) -> Any:
        """根据标签获取值"""
        labels = re.split(r"[,，]", label)
        values = [opt.value for opt in self.options if opt.label in labels]
        return values if len(values) > 1 else values[0] if values else None

    def get_option_labels(self) -> str:
        """获取所有标签"""
        return ", ".join(opt.label for opt in self.options)



from typing import Any, Dict, List, Optional

from quark import Request

from ..component.descriptions.descriptions import Descriptions as DescriptionsComponent
from ..component.table.column import Column
from ..component.table.column import Column as ColumnComponent


class ResolvesFields:

    # 请求
    request: Request = None

    # 字段
    fields: Optional[Any] = None

    # 表格列组件
    table_column: Column = None

    # 行内操作
    table_row_actions: Optional[Any] = None

    # 表格行内操作列标题
    table_action_column_title: str = "操作"

    # 表格行内操作列宽度
    table_action_column_width: Optional[int] = None

    def __init__(
        self,
        request: Request,
        fields: Optional[Any] = None,
        table_column: Column = None,
        table_row_actions: Optional[Any] = None,
        table_action_column_title: str = "操作",
        table_action_column_width: Optional[int] = None,
    ):
        self.request = request
        self.fields = fields
        self.table_column = table_column
        self.table_row_actions = table_row_actions
        self.table_action_column_title = table_action_column_title
        self.table_action_column_width = table_action_column_width

    def set_request(self, request) -> "ResolvesFields":
        """设置请求"""
        self.request = request
        return self

    def set_fields(self, fields) -> "ResolvesFields":
        """设置字段"""
        self.fields = fields
        return self

    def set_table_column(self, column) -> "ResolvesFields":
        """设置表格列组件"""
        self.table_column = column
        return self

    def set_table_row_actions(self, actions) -> "ResolvesFields":
        """设置行内操作"""
        self.table_row_actions = actions
        return self

    def set_table_action_column_title(self, title) -> "ResolvesFields":
        """设置行内操作列标题"""
        self.table_action_column_title = title
        return self

    def set_table_action_column_width(self, width) -> "ResolvesFields":
        """设置行内操作列宽度"""
        self.table_action_column_width = width
        return self

    def get_fields(self, include_when: bool = True) -> List[Any]:
        return self.find_fields(self.fields, include_when)

    def get_fields_without_when(self, include_when: bool = False) -> List[Any]:
        return self.find_fields(self.fields, include_when)

    def find_fields(self, fields: List[Any], include_when: bool) -> List[Any]:
        result = []
        for field in fields:
            if not hasattr(field, "body"):
                result.append(field)
                if include_when and hasattr(field, "when"):
                    result.extend(self.get_when_fields(field))
            else:
                sub_fields = self.find_fields(field.body, include_when)
                result.extend(sub_fields)
        return result

    def get_when_fields(self, item: Any) -> List[Any]:
        when_fields = []
        if not hasattr(item, "when") or getattr(item, "when") is None:
            return when_fields
        when_comp = getattr(item, "when")
        for option in when_comp.items:
            if option.body:
                if isinstance(option.body, list):
                    when_fields.extend(option.body)
                else:
                    when_fields.append(option.body)
        return when_fields

    def index_fields(self) -> List[Any]:
        return [f for f in self.get_fields() if f.is_shown_on_index()]

    def index_table_columns(self) -> List[ColumnComponent]:
        columns = []
        for field in self.index_fields():
            col = self.field_to_column(field)
            if col:
                columns.append(col)
        row_actions = self.table_row_actions
        if row_actions:
            action_col = (
                self.table_column.set_title(self.table_action_column_title)
                .set_width(self.table_action_column_width)
                .set_attribute("action")
                .set_value_type("option")
                .set_actions(row_actions)
                .set_fixed("right")
            )
            columns.append(action_col)
        return columns

    def field_to_column(self, field: Any) -> ColumnComponent:
        name = getattr(field, "name", "")
        label = getattr(field, "label", "")
        component = getattr(field, "component", "")
        align = getattr(field, "align", "")
        fixed = getattr(field, "fixed", None)
        editable = getattr(field, "editable", False)
        ellipsis = getattr(field, "ellipsis", None)
        copyable = getattr(field, "copyable", None)
        filters = getattr(field, "filters", None)
        order = getattr(field, "order", None)
        sorter = getattr(field, "sorter", None)
        span = getattr(field, "span", None)
        column_width = getattr(field, "column_width", None)
        options = None
        col = (
            ColumnComponent()
            .set_title(label)
            .set_attribute(name)
            .set_align(align)
            .set_fixed(fixed)
            .set_ellipsis(ellipsis)
            .set_copyable(copyable)
            .set_filters(filters)
            .set_order(order)
            .set_sorter(sorter)
            .set_span(span)
            .set_width(column_width)
        )

        if component == "idField":
            if getattr(field, "on_index_displayed", False):
                col.set_value_type("text")
            else:
                return None
        elif component == "hiddenField":
            return None
        elif component == "textField":
            col.set_value_type("text")
        elif component == "textAreaField":
            col.set_value_type("text")
        elif component == "editorField":
            col.set_value_type("text")
        elif component == "treeSelectField":
            options = field.get_options()
            col.set_value_type("treeSelect").set_field_props({"options": options})
        elif component == "cascaderField":
            options = field.get_options()
            col.set_value_type("cascader").set_field_props({"options": options})
        elif component == "selectField":
            options = field.get_options()
            col.set_value_type("select").set_field_props({"options": options})
            if isinstance(filters, bool) and filters:
                col.set_value_enum(field.get_value_enum())
        elif component == "checkboxField":
            options = field.get_options()
            col.set_value_type("checkbox").set_field_props({"options": options})
            if isinstance(filters, bool) and filters:
                col.set_value_enum(field.get_value_enum())
        elif component == "radioField":
            options = field.get_options()
            col.set_value_type("radio").set_field_props({"options": options})
            if isinstance(filters, bool) and filters:
                col.set_value_enum(field.get_value_enum())
        elif component == "switchField":
            options = field.get_options()
            col.set_value_type("select").set_value_enum(options)
        elif component == "imageField":
            col.set_value_type("image")
        elif component == "imagePickerField":
            col.set_value_type("image")
        elif component == "actionField":
            col.set_value_type("action")
        else:
            col.set_value_type(component)

        if editable:
            editable_api = self.request.url.path.replace("/index", "/editable")
            col.set_editable(component, options, editable_api)

        return col

    def creation_fields(self) -> List[Any]:
        return [f for f in self.get_fields() if f.is_shown_on_creation()]

    def creation_fields_without_when(self) -> List[Any]:
        return [f for f in self.get_fields_without_when() if f.is_shown_on_creation()]

    def creation_fields_within_components(self) -> List[Any]:
        return self.creation_form_fields_parser(self.fields)

    def update_fields(self) -> List[Any]:
        return [f for f in self.get_fields() if f.is_shown_on_update()]

    def update_fields_without_when(self) -> List[Any]:
        return [f for f in self.get_fields_without_when() if f.is_shown_on_update()]

    def update_fields_within_components(self) -> List[Any]:
        return self.update_form_fields_parser(self.fields)

    def detail_fields(self) -> List[Any]:
        return [f for f in self.get_fields() if f.is_shown_on_detail()]

    def detail_fields_within_components(
        self, init_api: Any, data: Dict[str, Any], detail_actions: List[Any]
    ) -> List[Any]:
        return self.detail_fields_parser(init_api, self.fields, data, detail_actions)

    def export_fields(self) -> List[Any]:
        return [
            f
            for f in self.get_fields()
            if getattr(f, "is_shown_on_export", lambda: False)()
        ]

    def import_fields(self) -> List[Any]:
        return [
            f
            for f in self.get_fields()
            if getattr(f, "is_shown_on_import", lambda: False)()
        ]

    def import_fields_without_when(self) -> List[Any]:
        return [f for f in self.get_fields_without_when() if f.is_shown_on_import()]

    def creation_form_fields_parser(self, fields: List[Any]) -> List[Any]:
        result = []
        for field in fields:
            if hasattr(field, "body"):
                body = getattr(field, "body")
                parsed_body = self.creation_form_fields_parser(body)
                setattr(field, "body", parsed_body)
                result.append(field)
            else:
                comp = getattr(field, "component", "")
                if "Field" in comp:
                    when = getattr(field, "when", None)
                    if when:
                        for item in when.items:
                            if item.body:
                                if isinstance(item.body, list):
                                    item.body = self.creation_form_fields_parser(
                                        item.body
                                    )
                                else:
                                    item.body.build_frontend_rules(
                                        self.request.url.path
                                    )
                    if field.is_shown_on_creation():
                        field.build_frontend_rules(self.request.url.path)
                        result.append(field)
                else:
                    result.append(field)
        return result

    def update_form_fields_parser(self, fields: List[Any]) -> List[Any]:
        result = []
        for field in fields:
            if hasattr(field, "body"):
                body = getattr(field, "body")
                parsed_body = self.update_form_fields_parser(body)
                setattr(field, "body", parsed_body)
                result.append(field)
            else:
                comp = getattr(field, "component", "")
                if "Field" in comp:
                    when = getattr(field, "when", None)
                    if when:
                        for item in when.items:
                            if item.body:
                                if isinstance(item.body, list):
                                    item.body = self.update_form_fields_parser(
                                        item.body
                                    )
                                else:
                                    item.body.build_frontend_rules(
                                        self.request.url.path
                                    )
                    if field.is_shown_on_update():
                        field.build_frontend_rules(self.request.url.path)
                        result.append(field)
                else:
                    result.append(field)
        return result

    def detail_fields_parser(
        self,
        init_api: Any,
        fields: List[Any],
        data: Dict[str, Any],
        detail_actions: List[Any],
    ) -> DescriptionsComponent:
        cols = []
        for field in fields:
            if hasattr(field, "body"):
                body = getattr(field, "body")
                parsed_body = self.detail_fields_parser(init_api, body, data)
                setattr(field, "body", parsed_body)
                cols.append(field)
            elif field.is_shown_on_detail():
                col = self.field_to_column(field)
                if col:
                    cols.append(col)

        return (
            DescriptionsComponent()
            .set_style({"padding": "24px"})
            .set_init_api(init_api)
            .set_title("")
            .set_column(2)
            .set_columns(cols)
            .set_data_source(data)
            .set_actions(detail_actions)
        )



# coding: utf-8
from quark import Quark, Response

# 创建对象
app = Quark()

# 配置数据库
app.config["DB_URL"] = "mysql://root:fK7xPGJi1gJfIief@localhost:3306/quarkcloud"


# 创建路由
@app.get("/")
def index():
    return Response(content="Hello World!", media_type="text/html")


if __name__ == "__main__":

    # 启动应用
    app.run("quark.__main__:app", host="0.0.0.0", port=3000, reload=True)



from pydantic import model_validator
from typing import Any, Dict, Optional
from ..component import Component


class ToolBar(Component):
    component: str = "toolBar"
    title: Optional[str] = None
    sub_title: Optional[str] = None
    description: Optional[str] = None
    search: Any = None
    actions: Any = None
    filter_: Any = None
    multiple_line: bool = False
    menu: Any = None
    tabs: Any = None

    @model_validator(mode="after")
    def init(self):
        self.set_key("toolBar")
        return self

    # 设置方法（链式调用）
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    def set_title(self, title: str):
        self.title = title
        return self

    def set_sub_title(self, subtitle: str):
        self.sub_title = subtitle
        return self

    def set_description(self, description: str):
        self.description = description
        return self

    def set_search(self, search: Any):
        self.search = search
        return self

    def set_action(self, callback: Any):
        # 由于 callback(p.action) 未实现，这里仅保留结构
        # 如果你有 action 的默认值或调用形式，可以添加进去
        return self

    def set_actions(self, actions: Any):
        self.actions = actions
        return self

    def set_filter(self, filter_: Any):
        self.filter_ = filter_
        return self

    def set_multiple_line(self, multiple_line: bool):
        self.multiple_line = multiple_line
        return self

    def set_menu(self, menu: Any):
        self.menu = menu
        return self

    def set_tabs(self, tabs: Any):
        self.tabs = tabs
        return self



from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field
from tortoise.models import Model
from tortoise.queryset import QuerySet

from quark import Message, Request

from ..component.form.form import Form
from ..component.table.column import Column
from ..component.table.search import Search
from ..component.table.table import Table
from ..component.table.tool_bar import ToolBar
from ..component.table.tree_bar import TreeBar
from .request.action import ActionRequest
from .request.detail import DetailRequest
from .request.edit import EditRequest
from .request.editable import EditableRequest
from .request.index import IndexRequest
from .request.update import UpdateRequest
from .resolves_fields import ResolvesFields
from .resource_create import ResourceCreate
from .resource_detail import ResourceDetail
from .resource_edit import ResourceEdit
from .resource_form import ResourceForm
from .resource_index import ResourceIndex


class Resource(
    BaseModel, ResourceIndex, ResourceForm, ResourceCreate, ResourceEdit, ResourceDetail
):
    """资源：增删改查"""

    # 页面标题
    title: str = Field(default="")

    # 页面副标题
    sub_title: str = Field(default="")

    # 是否显示返回图标
    back_icon: Any = Field(default=True)

    # 是否启用导出功能
    export: bool = Field(default=False)

    # 导出按钮的文本内容
    export_text: str = Field(default="导出")

    # 导出接口的路径
    export_path: str = Field(default="/api/admin/{resource}/export")

    # 每页显示的数据条数，默认为 10
    page_size: Any = Field(default=10)

    # 可选的每页条数选项
    page_size_options: List[int] = Field(default_factory=lambda: [10, 20, 50, 100])

    # 全局数据排序规则
    query_order: List[str] = Field(default_factory=lambda: [])

    # 列表页面的排序规则
    index_query_order: List[str] = Field(default_factory=lambda: [])

    # 导出数据时的排序规则
    export_query_order: List[str] = Field(default_factory=lambda: [])

    # 数据模型对象
    model: Any = Field(default=None)

    # 表单组件实例
    form: Form = Field(default_factory=lambda: Form())

    # 表格组件实例
    table: Table = Field(default_factory=lambda: Table())

    # 表格搜索组件实例
    table_search: Search = Field(default_factory=lambda: Search())

    # 表格列配置组件实例
    table_column: Column = Field(default_factory=lambda: Column())

    # 表格工具栏组件实例
    table_tool_bar: ToolBar = Field(default_factory=lambda: ToolBar())

    # 表格树形筛选组件实例
    table_tree_bar: TreeBar = Field(default_factory=lambda: TreeBar())

    # 表格标题后缀文字
    table_title_suffix: str = Field(default="列表")

    # 操作列标题
    table_action_column_title: str = Field(default="操作")

    # 操作列宽度，单位为像素
    table_action_column_width: Optional[int] = Field(default=None)

    # 表格数据轮询间隔时间（秒）
    table_polling: Any = Field(default=None)

    # 是否将表格数据转换为树形结构
    table_list_to_tree: bool = Field(default=False)

    async def init(self, request: Request) -> Any:
        """初始化"""
        return self

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        return []

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return []

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return []

    async def menu_items(self, request: Request) -> List[Dict[str, str]]:
        """菜单项定义"""
        return []

    async def get_model(self) -> Model:
        """获取模型实例"""
        if self.model is None:
            raise ValueError("Model not set")
        return self.model()

    async def before_exporting(self, request: Request, data: List[Dict]) -> List[Any]:
        """导出前处理"""
        return [item for item in data]

    async def after_exporting(self, request: Request, result: Any) -> Any:
        """导出后处理"""
        return result

    async def before_importing(
        self, request: Request, data: List[List[Any]]
    ) -> List[List[Any]]:
        """导入前处理"""
        return data

    async def before_editable(
        self, request: Request, id: Any, field: str, value: Any
    ) -> Optional[Exception]:
        """行内编辑前处理"""
        return None

    async def after_editable(
        self, request: Request, id: Any, field: str, value: Any
    ) -> Optional[Exception]:
        """行内编辑后处理"""
        return None

    async def before_action(
        self, request: Request, uri_key: str, query: QuerySet
    ) -> Optional[Exception]:
        """
        回调：在行为执行之前调用。
        """
        return None

    async def after_action(
        self, request: Request, uri_key: str, query: QuerySet
    ) -> Optional[Exception]:
        """
        回调：在行为执行之后调用。
        """
        return None

    async def query(self, request: Request) -> QuerySet:
        """
        全局查询
        """
        return self.model.all()

    async def index_query(self, request: Request) -> QuerySet:
        """
        列表查询
        """
        return await self.query(request)

    async def index_render(self, request: Request) -> Any:
        """列表页渲染"""

        # 获取搜索项
        searches = await self.searches(request)

        # 获取列表查询
        query = await self.index_query(request)

        # 获取列表页字段
        index_fields = ResolvesFields(
            request=request,
            fields=await self.fields(request),
        ).index_fields()

        index_data = await IndexRequest(
            request=request,
            resource=self,
            query=query,
            query_order=self.query_order,
            index_query_order=self.index_query_order,
            fields=index_fields,
            searches=searches,
            page_size=self.page_size,
            page_size_options=self.page_size_options,
        ).query_data()

        # 页面组件渲染
        return Message.success(
            "ok", await self.index_component_render(request, index_data)
        )

    async def creation_render(self, request: Request) -> Any:
        """列表页渲染"""

        data = await self.before_creating(request)

        # 页面组件渲染
        return Message.success(
            "ok", await self.creation_component_render(request, data)
        )

    async def store_render(self, request: Request) -> Any:
        """创建方法"""
        model = await self.get_model()

        data = await request.json()

        return await self.form_handle(request, model, data)

    async def edit_render(self, request: Request) -> Any:
        """编辑页渲染"""

        query = await self.query(request)

        fields = ResolvesFields(request, await self.fields(request)).update_fields()

        data = await EditRequest(
            request=request, resource=self, query=query, fields=fields
        ).fill_data()

        data = await self.before_editing(request, data)

        # 页面组件渲染
        return Message.success("ok", await self.update_component_render(request, data))

    async def edit_values_render(self, request: Request) -> Any:
        query = await self.query(request)

        fields = ResolvesFields(request, await self.fields(request)).update_fields()

        return await EditRequest(
            request=request, resource=self, query=query, fields=fields
        ).values()

    async def save_render(self, request: Request) -> Any:
        """更新方法"""
        fields = ResolvesFields(
            request, await self.fields(request)
        ).update_fields_without_when()

        model = await self.get_model()

        query = await self.query(request)

        return await UpdateRequest(
            request=request, resource=self, model=model, query=query, fields=fields
        ).handle()

    async def editable_render(self, request: Request) -> Any:
        """表格行内编辑"""
        return await EditableRequest(
            request=request, resource=self, query=await self.query(request)
        ).handle()

    async def action_render(self, request: Request) -> Any:
        """行为渲染"""
        return await ActionRequest(
            request=request,
            resource=self,
            query=await self.query(request),
            actions=await self.actions(request),
            fields=await self.fields(request),
        ).handle()

    async def action_values_render(self, request: Request) -> Any:
        """行为值渲染"""
        return await ActionRequest(
            request=request,
            resource=self,
            query=await self.query(request),
            actions=await self.actions(request),
            fields=await self.fields(request),
        ).values()

    async def detail_render(self, request: Request) -> Any:
        """详情页渲染"""

        query = await self.query(request)

        fields = ResolvesFields(request, await self.fields(request)).detail_fields()

        data = await DetailRequest(
            request=request, resource=self, query=query, fields=fields
        ).fill_data()

        data = await self.before_detail_showing(request, data)

        # 页面组件渲染
        return Message.success("ok", await self.detail_component_render(request, data))

    async def detail_values_render(self, request: Request) -> Any:
        """获取详情页值"""

        query = await self.query(request)

        fields = ResolvesFields(request, await self.fields(request)).detail_fields()

        return await DetailRequest(
            request=request, resource=self, query=query, fields=fields
        ).values()

    async def form_render(self, request: Request) -> Any:
        """列表页渲染"""

        data = await self.before_form_showing(request)

        # 页面组件渲染
        return Message.success(
            "ok", await self.creation_component_render(request, data)
        )



from typing import Any, Optional

from .action import Action


class Switch(Action):
    """
    表示一个开关组件，用于切换状态。
    """

    # 选中时的内容
    checked_children: Optional[Any] = None

    # 未选中时的内容
    unchecked_children: Optional[Any] = None

    # 字段名称
    field_name: Optional[Any] = None

    # 字段值
    field_value: Optional[Any] = None

    def __init__(self):
        self.action_type = "switch"

    def get_checked_children(self) -> Optional[Any]:
        """获取选中时的内容"""
        return self.checked_children

    def get_unchecked_children(self) -> Optional[Any]:
        """获取未选中时的内容"""
        return self.unchecked_children

    def get_field_name(self) -> Optional[Any]:
        """获取字段名称"""
        return self.field_name

    def get_field_value(self) -> Optional[Any]:
        """获取字段值"""
        return self.field_value



from typing import Any, Dict

from quark import Request

from ..utils import replace_last
from .resolves_actions import ResolvesActions
from .resolves_fields import ResolvesFields


class ResourceCreate:
    """
    创建表单
    """

    async def creation_api(self, request: Request) -> str:
        """
        获取创建表单提交接口地址
        """
        form_api = await self.form_api(request)
        if form_api:
            return form_api

        uri = request.url.path.split("/")
        last_segment = uri[-1]

        if last_segment == "index":
            return replace_last(request.url.path, "/index", "/store")
        elif last_segment == "form":
            return replace_last(request.url.path, "/form", "/store")

        return replace_last(request.url.path, "/create", "/store")

    async def creation_component_render(
        self, request: Request, data: Dict[str, Any]
    ) -> Any:
        """
        渲染创建页面组件
        """
        title = await self.form_title(request)
        actions = await self.actions(request)
        form_extra_actions = await ResolvesActions(
            request=request, actions=actions
        ).form_extra_actions()
        api = await self.creation_api(request)
        fields = ResolvesFields(
            request=request,
            fields=await self.fields(request),
        ).creation_fields_within_components()
        form_actions = await ResolvesActions(
            request=request, actions=actions
        ).form_actions()

        return await self.form_component_render(
            request, title, form_extra_actions, api, fields, form_actions, data
        )

    async def before_creating(self, request: Request) -> Dict[str, Any]:
        """
        创建页面显示前回调
        """
        return {}



from typing import Any, List, Optional

from quark import Request

from ...component.form.fields.cascader import Option
from .search import Search


class Cascader(Search):
    """级联选择搜索字段配置类"""

    cascader_options: List[Option] = None

    def __init__(self, column: str = "", name: str = ""):
        self.component = "cascaderField"
        return self

    def option(self, label: str, value: Any) -> Option:
        """
        创建一个选项对象

        :param label: 显示名称
        :param value: 值
        :return: Option 对象
        """
        return Option(value=value, label=label)

    def set_options(self, *options: Any) -> "Cascader":
        """
        设置可选项数据源

        支持以下几种调用方式：

        - set_options([Option(...), Option(...)])
        - set_options(options_list, parent_key, label_name, value_name)
        - set_options(options_list, level, parent_key, label_name, value_name)

        :param options: 可变参数，根据参数数量判断处理逻辑
        :return: Cascader 实例
        """
        if len(options) == 1 and isinstance(options[0], list):
            self.cascader_options = options[0]
        elif len(options) == 4:
            options_list, parent_key, label_name, value_name = options
            self.cascader_options = Option.list_to_options(
                options_list, 0, parent_key, label_name, value_name
            )
        elif len(options) == 5:
            options_list, level, parent_key, label_name, value_name = options
            self.cascader_options = Option.list_to_options(
                options_list, level, parent_key, label_name, value_name
            )
        else:
            raise ValueError("Unsupported number of arguments in set_options")

        return self

    def get_cascader_options(self) -> List[Option]:
        """获取级联选项数据"""
        return self.cascader_options

    def apply(self, request: Request, query: Any, value: Any) -> Any:
        """执行查询逻辑，子类可重写此方法"""
        return super().apply(request, query, value)

    def options(self, request: Request) -> Optional[Any]:
        """扩展属性选项"""
        return super().options(request)



from pydantic import BaseModel
from typing import Any, Optional
from ..component import Component

class Meta(Component):
    title: str = ""
    attribute: str = ""
    data_index: str = ""
    ellipsis: bool = False
    copyable: bool = False
    value_enum: Optional[Any] = None
    value_type: str = ""
    search: bool = False
    actions: bool = False

    def __init__(self, **data):
        super().__init__(**data)
        self.component = "meta"
        self.set_key("DEFAULT_KEY", False)

    # 初始化
    def init(self) -> 'Meta':
        self.component = "meta"
        self.set_key("DEFAULT_KEY", False)
        return self

    # Set style.
    def set_style(self, style: dict[str, Any]) -> 'Meta':
        self.style = style
        return self

    # 设置标题
    def set_title(self, title: str) -> 'Meta':
        self.title = title
        return self

    # 设置字段名称|字段的列名
    def set_attribute(self, attribute: str) -> 'Meta':
        self.data_index = attribute
        self.attribute = attribute
        return self

    # 设置是否自动缩略
    def set_ellipsis(self, ellipsis: bool) -> 'Meta':
        self.ellipsis = ellipsis
        return self

    # 设置是否支持复制
    def set_copyable(self, copyable: bool) -> 'Meta':
        self.copyable = copyable
        return self

    # 设置值的枚举，会自动转化把值当成 key 来取出要显示的内容
    def set_value_enum(self, value_enum: Any) -> 'Meta':
        value_enum_str = {}
        value_enum_int = {}

        if isinstance(value_enum, dict):
            for k, v in value_enum.items():
                if isinstance(k, str):
                    value_enum_str[k] = v
                elif isinstance(k, int):
                    value_enum_int[k] = v

        if value_enum_str:
            self.value_enum = value_enum_str
        if value_enum_int:
            self.value_enum = value_enum_int

        return self

    # 设置值的类型,"money" | "option" | "date" | "dateTime" | "time" | "text"| "index" | "indexBorder"
    def set_value_type(self, value_type: str) -> 'Meta':
        self.value_type = value_type
        return self

    # 在查询表单中不展示此项
    def set_search(self, search: bool) -> 'Meta':
        self.search = search
        return self

    def set_key(self, key: str, crypt: bool):
        # 这里简单处理，可根据实际需求实现
        pass


from typing import Any, List, Optional

from pydantic import Field, model_validator

from ..component import Component


class Drawer(Component):
    component: Optional[str] = Field(default="drawer")
    title: Any = Field(None, description="标题")
    body_style: Any = Field(None, alias="bodyStyle", description="Modal body 样式")
    closable: bool = Field(True, description="是否显示右上角的关闭按钮")
    content_wrapper_style: Any = Field(
        None,
        alias="contentWrapperStyle",
        description="可用于设置 Drawer 包裹内容部分的样式",
    )
    destroy_on_close: bool = Field(
        False, alias="destroyOnClose", description="关闭时销毁 Modal 里的子元素"
    )
    drawer_style: Any = Field(
        None, alias="drawerStyle", description="用于设置 Drawer 弹出层的样式"
    )
    footer_style: Any = Field(
        {"textAlign": "right"}, alias="footerStyle", description="抽屉页脚部件的样式"
    )
    height: int = Field(256, description="高度, 在 placement 为 top 或 bottom 时使用")
    keyboard: bool = Field(True, description="是否支持键盘 esc 关闭")
    mask: bool = Field(True, description="是否展示遮罩")
    mask_closable: bool = Field(
        True, alias="maskClosable", description="点击蒙层是否允许关闭"
    )
    mask_style: Any = Field(None, alias="maskStyle", description="遮罩样式")
    open: bool = Field(False, description="对话框是否可见")
    width: int = Field(256, description="宽度")
    z_index: int = Field(1000, alias="zIndex", description="设置 Modal 的 z-index")
    actions: List[Any] = Field([], description="弹窗行为")
    placement: str = Field(
        "right", description="抽屉的方向, top | right | bottom | left"
    )
    init_api: Optional[Any] = Field(None, alias="initApi", description="数据初始化接口")
    body: Any = Field([], description="容器控件里面的内容")  # 修改默认值为 []

    @model_validator(mode="after")
    def init(self):
        self.set_key("drawer")
        return self

    def set_style(self, style: Any):
        self.style = style
        return self

    def set_title(self, title: Any):
        self.title = title
        return self

    def set_body_style(self, style: Any):
        self.body_style = style
        return self

    def set_body(self, body: Any):
        self.body = body
        return self

    def set_closable(self, closable: bool):
        self.closable = closable
        return self

    def set_content_wrapper_style(self, style: Any):
        self.content_wrapper_style = style
        return self

    def set_destroy_on_close(self, destroy_on_close: bool):
        self.destroy_on_close = destroy_on_close
        return self

    def set_drawer_style(self, style: Any):
        self.drawer_style = style
        return self

    def set_footer_style(self, style: Any):
        self.footer_style = style
        return self

    def set_height(self, height: int):
        self.height = height
        return self

    def set_keyboard(self, keyboard: bool):
        self.keyboard = keyboard
        return self

    def set_mask(self, mask: bool):
        self.mask = mask
        return self

    def set_mask_closable(self, mask_closable: bool):
        self.mask_closable = mask_closable
        return self

    def set_mask_style(self, style: Any):
        self.mask_style = style
        return self

    def set_placement(self, placement: str):
        self.placement = placement
        return self

    def set_open(self, open_: bool):
        self.open = open_
        return self

    def set_width(self, width: int):
        self.width = width
        return self

    def set_z_index(self, z_index: int):
        self.z_index = z_index
        return self

    def set_actions(self, actions: List[Any]):
        self.actions = actions
        return self

    def set_init_api(self, api: Any):
        self.init_api = api
        return self



from typing import Any
from .base import Base


class Selects(Base):

    component: str = "selects"
    """
    组件名称
    """

    body: Any
    """
    组件内容
    """

    def set_body(self, body: Any):
        """
        设置组件内容。

        Args:
            body (Any): 组件内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.body = body
        return self



from typing import Any, Dict, List

from quark import Request, Resource, models, services
from quark.app import actions, searches
from quark.component.form import field
from quark.component.form.rule import Rule


class Role(Resource):
    """
    角色管理
    """

    async def init(self, request: Request):

        # 页面标题
        self.title = "角色"

        # 模型
        self.model = models.Role

        return self

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        return [
            field.id("id", "ID"),
            field.text("name", "名称")
            .set_rules(
                [
                    Rule.required("名称必须填写"),
                ]
            )
            .set_creation_rules(
                [
                    Rule.unique("roles", "name", "名称已存在"),
                ]
            )
            .set_update_rules(
                [
                    Rule.unique("roles", "name", "{id}", "名称已存在"),
                ]
            ),
            field.text("guard_name", "守卫名称").set_default_value("admin"),
            field.tree("menu_ids", "权限")
            .set_tree_data(await services.MenuService().get_list(), "pid", "id", "name")
            .only_on_forms(),
            field.datetime("created_at", "创建时间").only_on_index(),
            field.datetime("updated_at", "更新时间").only_on_index(),
            field.switch("status", "状态")
            .set_editable(True)
            .set_true_value("正常")
            .set_false_value("禁用")
            .set_default_value(True),
        ]

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("name", "名称"),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.CreateModal(self),
            actions.BatchDelete(),
            actions.EditModal(self),
            actions.Delete(),
            actions.FormSubmit(),
            actions.FormReset(),
            actions.FormBack(),
            actions.FormExtraBack(),
        ]

    async def before_editing(
        self, request: Request, data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        编辑页面显示前回调
        """

        # 获取用户菜单
        data["menu_ids"] = await services.RoleService().get_menu_ids_by_role_id(
            data["id"]
        )

        # 返回数据
        return data

    async def after_saved(self, request, id, data, result):
        await services.RoleService().save_menus_by_role_id(id, data["menu_ids"])



import json
from typing import Any, Dict, List, Optional

from pydantic import Field, model_validator

from ..component import Component


class ActivityConfig(Component):
    title: str = ""
    action: Any = None
    style: Dict[str, Any] = Field(default_factory=dict)


class Auth(Component):
    component: str = "auth"
    login_api: str = ""
    user_info_api: str = ""
    user_routes_api: str = ""
    redirect: str = ""
    logo: Any = None
    title: str = ""
    background_image_url: str = ""
    activity_config: Optional[ActivityConfig] = None
    values: Dict[str, Any] = Field(default_factory=dict)
    initial_values: Dict[str, Any] = Field(default_factory=dict)
    body: Any = None
    actions: List[Any] = Field(default_factory=list)

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    # 设置方法（链式调用）
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    def set_login_api(self, api: str):
        self.login_api = api
        return self

    def set_user_info_api(self, api: str):
        self.user_info_api = api
        return self

    def set_user_routes_api(self, api: str):
        self.user_routes_api = api
        return self

    def set_redirect(self, redirect: str):
        self.redirect = redirect
        return self

    def set_logo(self, logo: Any):
        self.logo = logo
        return self

    def set_title(self, title: str):
        self.title = title
        return self

    def set_background_image_url(self, url: str):
        self.background_image_url = url
        return self

    def set_activity_config(self, config: ActivityConfig):
        self.activity_config = config
        return self

    def set_initial_values(self, initial_values: Dict[str, Any]):
        data = initial_values.copy()

        if isinstance(self.body, list):
            for item in self.body:
                value = self._parse_initial_value(item, initial_values)
                if value is not None:
                    name = getattr(item, "name", "")
                    if name:
                        data[name] = value

        for k, v in data.items():
            if isinstance(v, str) and ("[" in v or "{" in v):
                try:
                    data[k] = json.loads(v)
                except Exception:
                    pass

        self.initial_values = data
        return self

    def set_body(self, body: Any):
        self.body = body
        return self

    def set_actions(self, actions: List[Any]):
        self.actions = actions
        return self

    # 私有方法
    def _parse_initial_value(self, item, initial_values):
        name = getattr(item, "name", "")
        value = getattr(item, "value", None)

        if not name:
            return None

        if hasattr(item, "default_value") and item.default_value is not None:
            value = item.default_value

        if name in initial_values:
            value = initial_values[name]

        return value



from typing import List

from .action import Action


class Step(Action):
    """
    表示一个步骤导航组件，用于处理“上一步”和“下一步”的操作。
    """

    # 按钮名称，默认为 ["上一步", "下一步"]
    name: List[str] = ("上一步", "下一步")

    def __init__(self):
        self.action_type = "step"
        self.name = ["上一步", "下一步"]



from typing import List, Optional

from tortoise.transactions import in_transaction

from quark.component.form.fields.checkbox import Option
from quark.models.menu_permission import MenuPermission
from quark.models.role_menu import RoleMenu
from quark.models.role_permission import RolePermission

from ..models import Role, UserRole


class RoleService:
    def __init__(self):
        pass

    async def get_info_by_id(self, role_id: int) -> Optional[Role]:
        role = await Role.get(id=role_id)
        return role

    async def update_role_data_scope(
        self, role_id: int, data_scope: int, department_ids: List[int]
    ) -> None:
        async with in_transaction() as connection:
            role = await Role.get(id=role_id)
            role.data_scope = data_scope
            await role.save(using_db=connection)

            if data_scope == 2:
                await self.add_department_to_role(role_id, department_ids, connection)
            else:
                await self.remove_role_departments(role_id)

    async def get_list(self) -> List[Option]:
        roles = await Role.filter(status=1).all()
        options = [Option(label=role.name, value=role.id) for role in roles]
        return options

    async def get_list_by_ids(self, ids: List[int]) -> List[Role]:
        roles = await Role.filter(id__in=ids).all()
        return roles

    async def save_roles_by_user_id(self, user_id: int, role_ids: List[int]):
        async with in_transaction() as connection:
            # 删除用户原有的角色
            await UserRole.filter(uid=user_id).delete()  # 移除 using_db 参数

            # 保存新的角色
            for role_id in role_ids:
                user_role = UserRole(uid=user_id, role_id=role_id)
                await user_role.save(using_db=connection)

    async def get_role_ids_by_user_id(self, user_id: int) -> List[int]:
        user_roles = await UserRole.filter(uid=user_id).all()
        return [user_role.role_id for user_role in user_roles]

    async def get_menu_ids_by_role_id(self, role_id: int) -> List[int]:
        role_menus = await RoleMenu.filter(role_id=role_id).all()
        return [role_menu.menu_id for role_menu in role_menus]

    async def save_menus_by_role_id(
        self, role_id: int, menu_ids: List[int]
    ) -> List[int]:
        await RoleMenu.filter(role_id=role_id).delete()
        await RolePermission.filter(role_id=role_id).delete()
        for menu_id in menu_ids:
            role_menu = RoleMenu(role_id=role_id, menu_id=menu_id)
            await role_menu.save()
            menu_permissions = await MenuPermission.filter(menu_id=menu_id).all()
            for menu_permission in menu_permissions:
                role_permission = RolePermission(
                    role_id=role_id, permission_id=menu_permission.permission_id
                )
                await role_permission.save()

    async def add_department_to_role(
        self, role_id: int, department_ids: List[int], connection=None
    ):
        # 这里你需要实现具体逻辑，比如插入 RoleDepartment 关联表
        pass

    async def remove_role_departments(self, role_id: int, connection=None):
        # 这里你需要实现具体逻辑，比如删除 RoleDepartment 关联表里对应数据
        pass
